//****************************************************
//
//	project		PAFEE
//!	@file:		CharacterBuffer.cpp
//!	@author:	
//!	@brief:		Ring buffer for character stream
//
//****************************************************
//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "PafeeCommon.h"
#include "pafee_string.h"
#include "PafeeCharacterBuffer.h"

#ifdef PAFEE_UNITTEST
#include "../Utility/PafeeCoverageRecorder.h"
static Pafee::CoverageRecorder g_coverage;
#define COVERAGE_RECORD(LINE)		g_coverage.Record(LINE)
#else
#define COVERAGE_RECORD(LINE)		
#endif

using namespace Pafee;


//----------------------------------------------------------------------------
//!	Constructor
//! @return: none
//----------------------------------------------------------------------------
CharacterBuffer::CharacterBuffer(int capacity/*=0*/, Allocator* pAllocator/*=NULL*/) 
{
	m_isInitialized = false;
	Initialize(capacity, pAllocator);
	COVERAGE_RECORD(__LINE__);
}

//----------------------------------------------------------------------------
//!	Destructor
//! @return: none
//----------------------------------------------------------------------------
CharacterBuffer::~CharacterBuffer()
{
	Deallocate(m_pCharacterBuffer);
	COVERAGE_RECORD(__LINE__);
}

//----------------------------------------------------------------------------
//!	Initialize character buffer
//! @return: Result(true:success false: failure)
//----------------------------------------------------------------------------
bool CharacterBuffer::Initialize(
								int capacity,					///< [in] character buffer capacity
								Allocator* pAllocator/*=NULL*/	///< [in] memory allocator
								)
{
	if(!m_isInitialized)
	{
#ifdef PAFEE_STATISTICS_ENABLE
		pafee_memset(&m_statistics, 0, sizeof(m_statistics));
#endif
		SetAllocator(pAllocator);
		
		// allocate caharacter buffer
		m_pCharacterBuffer = (char*) Allocate(capacity+1);

		if(m_pCharacterBuffer == NULL)
		{
			COVERAGE_RECORD(__LINE__);
			return false;
		}

		m_endPoint = (unsigned long)m_pCharacterBuffer + capacity+1;
		m_capacity = capacity;
		m_readPoint = (unsigned long)m_pCharacterBuffer;
		m_writePoint = (unsigned long)m_pCharacterBuffer;

		m_isInitialized = true;
		COVERAGE_RECORD(__LINE__);
	}
	COVERAGE_RECORD(__LINE__);
	return true;
}

//----------------------------------------------------------------------------
//!	Get current element count
//! @return: count of elements
//----------------------------------------------------------------------------
inline int CharacterBuffer::GetCount(void)
{
	if(m_readPoint>m_writePoint)
	{
		COVERAGE_RECORD(__LINE__);
		return ((m_endPoint - m_readPoint) + (m_writePoint - (unsigned long)m_pCharacterBuffer));
	}
	COVERAGE_RECORD(__LINE__);
	return m_writePoint - m_readPoint;
}

//----------------------------------------------------------------------------
//!	Check buffer is empty or not
//! @return: true:empty false: not empty
//----------------------------------------------------------------------------
inline bool CharacterBuffer::IsEmpty(void)
{
	COVERAGE_RECORD(__LINE__);
	return (m_readPoint == m_writePoint)? true : false;
}

//----------------------------------------------------------------------------
//!	Enqueue
//! @return: Result(true:success false: failure)
//----------------------------------------------------------------------------
inline bool CharacterBuffer::Enqueue(char data)
{
	if((m_readPoint>m_writePoint)? (m_writePoint + 1 < m_readPoint) : (m_writePoint - m_readPoint < m_capacity))
	{
		*(char*)m_writePoint = data;
		if(++m_writePoint >= m_endPoint)
		{
			m_writePoint = (unsigned long)m_pCharacterBuffer;
			COVERAGE_RECORD(__LINE__);
		}
		STATISTICS_INC(m_statistics.addCount);
		STATISTICS_MAX(m_statistics.maxElementCount, (unsigned long) GetCount());
		COVERAGE_RECORD(__LINE__);
	}
	else
	{
		STATISTICS_INC(m_statistics.overflowCount);
		COVERAGE_RECORD(__LINE__);
		return false;
	}
	COVERAGE_RECORD(__LINE__);
	return true;
}

//----------------------------------------------------------------------------
//!	Dequeue
//! @return: Result(true:success false: failure)
//----------------------------------------------------------------------------
inline bool CharacterBuffer::Dequeue(char* pData)
{
	if(m_writePoint != m_readPoint)
	{
		*pData = *(char*)m_readPoint;
		if(++m_readPoint >= m_endPoint)
		{
			m_readPoint = (unsigned long)m_pCharacterBuffer;
			COVERAGE_RECORD(__LINE__);
		}
		COVERAGE_RECORD(__LINE__);
		return true;
	}
	COVERAGE_RECORD(__LINE__);
	return false;
}

//----------------------------------------------------------------------------
//!	Clear character buffer
//! @return: void
//----------------------------------------------------------------------------
void CharacterBuffer::ClearAll(void)
{
	m_readPoint = (unsigned long)m_pCharacterBuffer;
	m_writePoint = (unsigned long)m_pCharacterBuffer;
	COVERAGE_RECORD(__LINE__);
}

//----------------------------------------------------------------------------
//!	Get statistics info
//! @return: void
//----------------------------------------------------------------------------
#ifdef PAFEE_STATISTICS_ENABLE
void CharacterBuffer::GetStatistics(CharacterBuffer::STATISTICS* pStatistics)
{
	m_statistics.currentElementCount = GetCount();
	pafee_memcpy(pStatistics, &m_statistics, sizeof(m_statistics));
	COVERAGE_RECORD(__LINE__);
}
#endif

#ifdef PAFEE_UNITTEST
//----------------------------------------------------------------------------
//!	Unit test
//! @return: void
//----------------------------------------------------------------------------
void CharacterBuffer::UnitTest(void)
{
	const int TEST_CHARACTER_BUFFER_CAPACITY = 10;

	CharacterBuffer	buffer(TEST_CHARACTER_BUFFER_CAPACITY);
	char			pData[TEST_CHARACTER_BUFFER_CAPACITY*2];
	char			pOutput[TEST_CHARACTER_BUFFER_CAPACITY*2];
	int 			i;
	int 			halfCapacity = TEST_CHARACTER_BUFFER_CAPACITY / 2;

	// statistics info
	unsigned long			addCount = 0;
	unsigned long			overflowCount = 0;
#ifdef PAFEE_STATISTICS_ENABLE
	STATISTICS		statistics;
#endif

	// initialize test data
	for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY*2; i++)
	{
		pData[i] = i;
	}

	//
	// test Initialize
	//
	assert(buffer.Initialize(TEST_CHARACTER_BUFFER_CAPACITY, NULL) == true);

	//
	// test GetCount/IsEmpty/Enqueue/Dequeue
	//
	{
		int count = 0;
		assert(buffer.GetCount() == 0);	// write_point == read_point
		assert(buffer.IsEmpty());
		pOutput[0]='\0';
		assert(!buffer.Dequeue(&pOutput[0]));

		// Enqueue test

		for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY; i++)
		{
			assert(buffer.Enqueue(pData[i]));
			count++;
			addCount++;
			assert(buffer.GetCount() == count);	// write_point > read_point
		}
		assert(!buffer.IsEmpty());

		assert(!buffer.Enqueue(pData[0]));
		overflowCount++;


		// Dequeue test

		for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY; i++)
		{
			assert(buffer.Dequeue(&pOutput[i]));
			count--;
			assert(buffer.GetCount() == count);	// write_point > read_point
			assert(pOutput[i] == i);
		}
		assert(buffer.IsEmpty());

		assert(!buffer.Dequeue(&pOutput[0]));


		// Enqueue and Dequeue test

		for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY; i++)
		{
			assert(buffer.Enqueue(pData[i]));
			addCount++;
			count++;
		}
		assert(!buffer.IsEmpty());

		assert(!buffer.Enqueue(pData[0]));
		overflowCount++;

		for(i=0; i<halfCapacity; i++)
		{
			assert(buffer.Dequeue(&pOutput[i]));
			count--;
			assert(pOutput[i] == i);
		}
		assert(!buffer.IsEmpty());

		for(i=0; i<halfCapacity; i++)
		{
			assert(buffer.Enqueue(pData[TEST_CHARACTER_BUFFER_CAPACITY+i]));
			addCount++;
			count++;
			assert(buffer.GetCount() == count);	// write_point < read_point
		}
		assert(!buffer.IsEmpty());

		assert(!buffer.Enqueue(pData[0]));
		overflowCount++;

		for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY; i++)
		{
			assert(buffer.Dequeue(&pOutput[i]));
			count--;
			assert(buffer.GetCount() == count); // (write_point < read_point) -> (write_point > read_point)
			assert(pOutput[i] == i+halfCapacity);
		}
		assert(buffer.IsEmpty());

		assert(!buffer.Dequeue(&pOutput[0]));
	}

	//
	// test ClearAll
	//
	{
		for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY; i++)
		{
			buffer.Enqueue(pData[i]);
			addCount++;
		}
		buffer.ClearAll();
		assert(buffer.IsEmpty());
		assert(!buffer.Dequeue(&pOutput[0]));
		for(i=0; i<TEST_CHARACTER_BUFFER_CAPACITY; i++)
		{
			assert(buffer.Enqueue(pData[i]));
			addCount++;
		}
		assert(!buffer.Enqueue(pData[0]));
		overflowCount++;
	}

#ifdef PAFEE_STATISTICS_ENABLE
	//
	// test GetStatistics
	//
	{
		buffer.ClearAll();
		for(i=0; i<halfCapacity; i++)
		{
			buffer.Enqueue(pData[i]);
			addCount++;
		}

		buffer.GetStatistics(&statistics);
		printf("max:%ld\n", statistics.maxElementCount);
		assert(statistics.addCount == addCount);
		assert(statistics.overflowCount == overflowCount);
		assert(statistics.maxElementCount == (unsigned long)TEST_CHARACTER_BUFFER_CAPACITY);
		assert(statistics.currentElementCount == (unsigned long)halfCapacity);
	}
#endif // PAFEE_STATISTICS_ENABLE
	g_coverage.Show();
}
#endif // PAFEE_UNITTEST
