//****************************************************
//
//    project   Pafee
//!   @file:    PafeeListT.h
//!   @author:  M.Mori
//!   @brief:   ListT Hedder File
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef PAFEE_INCLUDE_PAFEELISTT_H_
#define PAFEE_INCLUDE_PAFEELISTT_H_
//
#include <new>
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_string.h"
#include <limits.h>
//
#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{ 

//! ListT
template<class T>
class ListT : public Object
{
#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct
	{
		int addCount;
		int deleteCount;
		int maxElementCount;
		int presentElementCount;
	} STATISTICS;
	void GetStatistics(STATISTICS* pStatistics)
	{
		pafee_memcpy ( pStatistics, &m_Statistics, sizeof (STATISTICS) );
	}
private:
	STATISTICS m_Statistics;
#endif // PAFEE_STATISTICS_ENABLE

protected:
	struct NODE_BOX
	{
		NODE_BOX()
		{
			m_pPrev=NULL;
			m_pNext=NULL;
		}
		NODE_BOX(const T& Data)
		{
			m_pPrev=NULL;
			m_pNext=NULL;
			m_Data = Data;
		}
		NODE_BOX& operator = (const T& Data)
		{
			m_Data = Data;
			return *this;
		}
		virtual ~NODE_BOX(){};
		NODE_BOX* m_pPrev;
		NODE_BOX* m_pNext;
		T m_Data;
	};
public:
	typedef NODE_BOX* POSITION;

public:
	ListT(
		Allocator* pAllocator=NULL ///< [in] Memory Allocator
	)
	{
		Initialize(pAllocator);
	}

	ListT(
		const ListT& lstc, ///< [in] initial value of List
		Allocator* pAllocator=NULL ///< [in] Memory Allocator
	)
	{
		Initialize(pAllocator);
		POSITION pFind = lstc.m_pStart;
		while(pFind != NULL)
		{
			this->AddBack(pFind->m_Data);
			pFind = pFind->m_pNext;
		}
		STATISTICS_SET(m_Statistics.addCount,lstc.m_Statistics.addCount);
		STATISTICS_SET(m_Statistics.deleteCount,lstc.m_Statistics.deleteCount);
		STATISTICS_SET(m_Statistics.maxElementCount,lstc.m_Statistics.maxElementCount);
		STATISTICS_SET(m_Statistics.presentElementCount,lstc.m_Statistics.presentElementCount);
	}

	virtual ~ListT()
	{
		NODE_BOX* p;
		NODE_BOX* pdel;
		for( p=m_pEnd; p!=NULL; )
		{
			pdel = p;
			p = p->m_pPrev;
			pdel->~NODE_BOX();
			Deallocate ( (void*)pdel );
		}
	}

	bool Initialize(
		Allocator* pAllocator=NULL ///< [in] Memory Allocator
	)
	{
		SetAllocator(pAllocator);
		m_pStart = NULL;
		m_pEnd = NULL;
		m_elementNum = 0;
		STATISTICS_SET(m_Statistics.addCount,0);
		STATISTICS_SET(m_Statistics.deleteCount,0);
		STATISTICS_SET(m_Statistics.maxElementCount,0);
		STATISTICS_SET(m_Statistics.presentElementCount,0);
		return true;
	}

	ListT& operator =(
		const ListT& lstc ///< [in] Copied ListT
	)
	{
		this->RemoveAll();
		Initialize(lstc.m_pAllocator);
		POSITION pFind = lstc.m_pStart;
		while(pFind != NULL)
		{
			this->AddBack(pFind->m_Data);
			pFind = pFind->m_pNext;
		}
		STATISTICS_SET(m_Statistics.addCount,lstc.m_Statistics.addCount);
		STATISTICS_SET(m_Statistics.deleteCount,lstc.m_Statistics.deleteCount);
		STATISTICS_SET(m_Statistics.maxElementCount,lstc.m_Statistics.maxElementCount);
		STATISTICS_SET(m_Statistics.presentElementCount,lstc.m_Statistics.presentElementCount);
		return *this;
	}
	int GetCount(void)
	{
		return m_elementNum;
	}
	bool IsEmpty(void)
	{
		return !m_elementNum;
	}

	bool AddFront(
		const T& data ///< [in] Set value
	)
	{
		NODE_BOX* pNew = (NODE_BOX*)Allocate(sizeof(NODE_BOX));
		pNew = new (pNew)NODE_BOX(data);
		if(pNew == NULL)
		{
			return false;
		}
		if(m_pStart == NULL)
		{
			m_pStart = m_pEnd = pNew;
		}
		else
		{
			pNew->m_pNext = m_pStart;
			m_pStart->m_pPrev = pNew;
			m_pStart = pNew;
		}
		m_elementNum++;
		STATISTICS_INC(m_Statistics.addCount);
		STATISTICS_INC(m_Statistics.presentElementCount);
		STATISTICS_MAX(m_Statistics.maxElementCount,m_Statistics.presentElementCount);
		return true;
	}

	bool AddBack(
		const T& data ///< [in] Set value
	)
	{
		NODE_BOX* pNew = (NODE_BOX*)Allocate(sizeof(NODE_BOX));
		pNew = new (pNew)NODE_BOX(data);
		if(pNew == NULL)
		{
			return false;
		}
		if( m_pStart == NULL)
		{
			m_pStart = m_pEnd = pNew;
		}
		else
		{
			pNew->m_pPrev = m_pEnd;
			m_pEnd->m_pNext = pNew;
			m_pEnd = pNew;
		}
		m_elementNum++;
		STATISTICS_INC(m_Statistics.addCount);
		STATISTICS_INC(m_Statistics.presentElementCount);
		STATISTICS_MAX(m_Statistics.maxElementCount,m_Statistics.presentElementCount);
		return true;
	}

	T& GetFront(void)
	{
		return m_pStart->m_Data;
	}

	T& GetBack(void)
	{
		return m_pEnd->m_Data; 
	}

	POSITION GetFrontPosition(void)
	{
		return m_pStart;
	}

	POSITION GetBackPosition(void)
	{
		return m_pEnd;
	}

	T& GetNext(
		POSITION& pos ///< [in] Position
	)
	{
		NODE_BOX* pTarget = pos;
		pos = (POSITION) pTarget->m_pNext;
		return pTarget->m_Data;
	}

	T& GetPrevious(
		POSITION &pos ///< [in/out] Previous Node Pointer
	)
	{
		NODE_BOX* pTarget = pos;
		pos = (POSITION) pTarget->m_pPrev;
		return pTarget->m_Data;
	}
	
	bool GetAt(
		POSITION& pos, ///< [in] Position
		T& Data  ///< [in] The data that was gotten
	)
	{
		if(pos != NULL)
		{
			Data = pos->m_Data;
			return true;
		}
		return false;
	}
	bool SetAt(
		POSITION& pos, ///< [in] Position
		const T& Data  ///< [in] data to set
	)
	{
		if(pos != NULL)
		{
			pos->m_Data = Data;
			return true;
		}
		return false;
	}

	bool RemoveFront(void)
	{
		NODE_BOX* p;
		if(m_pStart == NULL)
		{
			return false;
		}
		p = m_pStart;
		m_pStart = m_pStart->m_pNext;
		if(m_pStart != NULL)
		{
			m_pStart->m_pPrev = NULL;
		}
		else
		{
			m_pEnd = NULL;
		}
		p->~NODE_BOX();
		Deallocate((void*)p);
		m_elementNum--;
		STATISTICS_INC(m_Statistics.deleteCount);
		STATISTICS_DEC(m_Statistics.presentElementCount);
		return true;
	}

	bool RemoveBack(void)
	{
		NODE_BOX* p;
		if(m_pEnd == NULL)
		{
			return false;
		}
		p = m_pEnd;
		m_pEnd = m_pEnd->m_pPrev;
		if(m_pEnd != NULL)
		{
			m_pEnd->m_pNext = NULL;
		}
		else
		{
			m_pStart = NULL;
		}
		p->~NODE_BOX();
		Deallocate((void*)p);
		m_elementNum--;
		STATISTICS_INC(m_Statistics.deleteCount);
		STATISTICS_DEC(m_Statistics.presentElementCount);
		return true;
	}

	void RemoveAt(
		POSITION pos ///< [in] Position
	)
	{
		if(pos->m_pPrev != NULL)
		{
			pos->m_pPrev->m_pNext = pos->m_pNext;
		}
		if(pos->m_pNext != NULL)
		{
			pos->m_pNext->m_pPrev = pos->m_pPrev;
		}
		if(pos == m_pStart)
		{
			m_pStart = pos->m_pNext;
		}
		if(pos == m_pEnd)
		{
			m_pEnd = pos->m_pPrev;;
		}
		pos->~NODE_BOX();
		Deallocate((void*)pos);
		m_elementNum--;
		if(m_elementNum == 0)
		{
			m_pStart = NULL;
			m_pEnd = NULL;
		}
		STATISTICS_INC(m_Statistics.deleteCount);
		STATISTICS_DEC(m_Statistics.presentElementCount);
		return;
	}

	void RemoveAll()
	{
		while(m_pStart != NULL)
		{
			RemoveFront();
		}
	}

protected:
	NODE_BOX* m_pStart;
	NODE_BOX* m_pEnd;
	int m_elementNum;
};

}

#endif
