//****************************************************
//
//    project   Pafee
//!   @file:    PafeeMultiMapT.h
//!   @author:  M.Tanaka
//!   @brief:   MultiMapT Hedder File
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include <new>
#include <limits.h>
#include "PafeeList.h"
#include "pafee_string.h"
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "PafeeAllocator.h"

#ifndef PAFEE_MULTI_MAPT
#define PAFEE_MULTI_MAPT

namespace Pafee
{

//! MultiMapT
	template<class T>
	class MultiMapT : public Object
	{

#ifdef PAFEE_STATISTICS_ENABLE
	public:
		typedef struct
		{
			int addCount;
			int acqCount;
			int deleteCount;
			int maxElementCount;
			int collisionCount;
			int minListCount;
			int maxListCount;
			int presentElementCount;
		} STATISTICS;
private:
		STATISTICS m_Statistics;
#endif // PAFEE_STATISTICS_ENABLE

	public:
		struct POSITION
		{
			List::POSITION pos;
			int m_positionIndex;
			const char* m_pCharkey;
			int m_numberKey;
		};

	public:
		MultiMapT(
			int hashSize=0,///[in] Hash table size
			Allocator* pAllocator=NULL ///[in] Memory allocator
		)
		{
			m_size =0;
			m_pHashTable = NULL;
			m_indexCount =0;
			Initialize( hashSize, pAllocator);
			STATISTICS_SET(m_Statistics.addCount,0);
			STATISTICS_SET(m_Statistics.acqCount,0);
			STATISTICS_SET(m_Statistics.deleteCount,0);
			STATISTICS_SET(m_Statistics.maxElementCount,0);
			STATISTICS_SET(m_Statistics.collisionCount,0);
			STATISTICS_SET(m_Statistics.minListCount,0);
			STATISTICS_SET(m_Statistics.maxListCount,0);
			STATISTICS_SET(m_Statistics.presentElementCount,0);
		}

		MultiMapT(
			const MultiMapT& mmpc, ///< [in] initial value of MultiMap
			Allocator* pAllocator=NULL ///< [in] Memory Allocator
		)
		{
			m_size =0;
			m_pHashTable = NULL;
			m_indexCount =0;
			m_writeIndex=0;
			m_pWriteBuffer=NULL;
			m_writeBufferSize=0;
			Initialize( mmpc.m_hashSize, pAllocator);
			for(int i=0; i<m_hashSize; i++)
			{
				m_pHashTable[i] = mmpc.m_pHashTable[i];
			}
			m_size = mmpc.m_size;
			STATISTICS_SET(m_Statistics.addCount,mmpc.m_Statistics.addCount);
			STATISTICS_SET(m_Statistics.acqCount,mmpc.m_Statistics.acqCount);
			STATISTICS_SET(m_Statistics.deleteCount,mmpc.m_Statistics.deleteCount);
			STATISTICS_SET(m_Statistics.maxElementCount,mmpc.m_Statistics.maxElementCount);
			STATISTICS_SET(m_Statistics.collisionCount,mmpc.m_Statistics.collisionCount);
			STATISTICS_SET(m_Statistics.minListCount,mmpc.m_Statistics.minListCount);
			STATISTICS_SET(m_Statistics.maxListCount,mmpc.m_Statistics.maxListCount);
			STATISTICS_SET(m_Statistics.presentElementCount,mmpc.m_Statistics.presentElementCount);
		}

		virtual ~MultiMapT()
		{
			if ( m_pHashTable != NULL )
			{
				for(int i=0; i<m_hashSize; i++)
				{
					(&(m_pHashTable[i]))->~List();
				}
				Deallocate(m_pHashTable);
			}
		}

		MultiMapT& operator =(
			const MultiMapT& mmpc ///< [in] Copied MultiMapT
		)
		{
			this->RemoveAll();
			if ( m_pHashTable != NULL )
			{
				for(int i=0; i<m_hashSize; i++)
				{
					(&(m_pHashTable[i]))->~List();
				}
				Deallocate(m_pHashTable);
			}
			m_size =0;
			m_pHashTable = NULL;
			m_indexCount =0;
			m_writeIndex=0;
			m_pWriteBuffer=NULL;
			m_writeBufferSize=0;
			Initialize( mmpc.m_hashSize, m_pAllocator);
			for(int i=0; i<m_hashSize; i++)
			{
				m_pHashTable[i] = mmpc.m_pHashTable[i];
			}
			m_size = mmpc.m_size;
			STATISTICS_SET(m_Statistics.addCount,mmpc.m_Statistics.addCount);
			STATISTICS_SET(m_Statistics.acqCount,mmpc.m_Statistics.acqCount);
			STATISTICS_SET(m_Statistics.deleteCount,mmpc.m_Statistics.deleteCount);
			STATISTICS_SET(m_Statistics.maxElementCount,mmpc.m_Statistics.maxElementCount);
			STATISTICS_SET(m_Statistics.collisionCount,mmpc.m_Statistics.collisionCount);
			STATISTICS_SET(m_Statistics.minListCount,mmpc.m_Statistics.minListCount);
			STATISTICS_SET(m_Statistics.maxListCount,mmpc.m_Statistics.maxListCount);
			STATISTICS_SET(m_Statistics.presentElementCount,mmpc.m_Statistics.presentElementCount);
			return *this;
		}

		bool Initialize(
			int hashSize,///[in] Hash table size
			Allocator* pAllocator=NULL ///[in] Memory allocator
		)
		{
			SetAllocator(pAllocator);
			if(hashSize == 0)
			{
				return false;
			}
			if(m_pHashTable != NULL)
			{
				return false;
			}
			m_hashSize = hashSize;
			m_pHashTable = (List*)Allocate(sizeof(List) * m_hashSize);
			if(!m_pHashTable)
			{
				return false;
			}
			for(int i=0; i<m_hashSize; i++)
			{
				new (&(m_pHashTable[i]))List(pAllocator);
			}
			return true;
		}

		int GetCount(void)
		{
			return m_size;
		}

		bool IsEmpty(void)
		{
			return !m_size;
		}

		bool Set(
			const char* szKey,///[in] Key
			const T& data///[in] Set data
		)
		{
			unsigned int index = Hash( szKey );
			char* pSetBuffer;
			int bufferSize;
			if(m_pHashTable[index].GetCount() > 0)
			{
				STATISTICS_INC(m_Statistics.collisionCount);
			}
			pSetBuffer = this->SetData(szKey, data, &bufferSize);
			m_pHashTable[index].AddBack( pSetBuffer, bufferSize);
			m_size++;
			STATISTICS_INC(m_Statistics.addCount);
			STATISTICS_INC(m_Statistics.presentElementCount);
			STATISTICS_MAX(m_Statistics.maxElementCount,m_Statistics.presentElementCount);
			return true;
		}

		bool Set(
			int key,///[in] Key
			const T& data///[in] Set data
		)
		{
			unsigned int index = Hash( key );
			char* pSetBuffer;
			int bufferSize;
			if(m_pHashTable[index].GetCount() > 0)
			{
				STATISTICS_INC(m_Statistics.collisionCount);
			}
			pSetBuffer = this->SetData(key, data, &bufferSize);
			m_pHashTable[index].AddBack( pSetBuffer, bufferSize);
			m_size++;
			STATISTICS_INC(m_Statistics.addCount);
			STATISTICS_INC(m_Statistics.presentElementCount);
			STATISTICS_MAX(m_Statistics.maxElementCount,m_Statistics.presentElementCount);
			return true;
		}

		POSITION GetStartPosition(void)
		{
			POSITION myPosition;
			for(int index=0;index<m_hashSize;index++)
			{
				List::POSITION pos = m_pHashTable[index].GetFrontPosition();
				if ( pos != NULL )
				{
					myPosition.pos = pos;
					myPosition.m_positionIndex = index;
					myPosition.m_pCharkey = NULL;
					myPosition.m_numberKey = -1;
					return myPosition;
				}
			}
			myPosition.pos = NULL;
			myPosition.m_positionIndex = 0;
			myPosition.m_pCharkey = NULL;
			myPosition.m_numberKey = -1;
			return myPosition;
		}

		POSITION GetStartPosition(
			const char* szKey///[in] Key
		)
		{
			POSITION myPosition;
			char* keyChar;
			int keyNum;
			char* pData;
			int length;
			unsigned int index = Hash( szKey );
			List::POSITION pos = m_pHashTable[index].GetFrontPosition();
			List::POSITION prepos;
			while(pos)
			{
				prepos = pos;
				pData = (char*)m_pHashTable[index].GetNext(pos,&length);
				keyChar = this->GetKey(pData,&keyNum);
				if(keyChar != NULL)
				{
					if ( pafee_strcmp(keyChar, szKey ) == 0 )
					{
						myPosition.pos = prepos;
						myPosition.m_positionIndex = index;
						myPosition.m_pCharkey = keyChar;
						myPosition.m_numberKey = -1;
						return myPosition;
					}
				}
			}
			myPosition.pos = NULL;
			myPosition.m_positionIndex = 0;
			myPosition.m_pCharkey = szKey;
			myPosition.m_numberKey = -1;
			return myPosition;
		}

		POSITION GetStartPosition(
			int key ///[in] Key
		)
		{
			POSITION myPosition;
			char* keyChar;
			int keyNum;
			char* pData;
			int length;
			unsigned int index = Hash( key );
			List::POSITION pos = m_pHashTable[index].GetFrontPosition();
			List::POSITION prepos;
			while(pos)
			{
				prepos = pos;
				pData = (char*)m_pHashTable[index].GetNext(pos,&length);
				keyChar = this->GetKey(pData,&keyNum);
				if(keyChar == NULL)
				{
					if (  keyNum == key  )
					{
						myPosition.pos = prepos;
						myPosition.m_positionIndex = index;
						myPosition.m_pCharkey = NULL;
						myPosition.m_numberKey = keyNum;
						return myPosition;
					}
				}
			}
			myPosition.pos = NULL;
			myPosition.m_positionIndex = 0;
			myPosition.m_pCharkey = NULL;
			myPosition.m_numberKey = key;
			return myPosition;
		}

		bool GetNext(
			POSITION& pos, ///[in] Position
			T& data///[out] Get data
		)
		{
			char* pData;
			char* keyChar;
			int keyNum;
			int length;
			if(pos.pos == NULL)
			{
				pos.m_positionIndex++;
				pos.pos = m_pHashTable[pos.m_positionIndex].GetFrontPosition();
			}
			for( int j=pos.m_positionIndex;j<m_hashSize;j++)
			{
				while(pos.pos)
				{
					pData = (char*)m_pHashTable[j].GetNext(pos.pos,&length);
					if ( pData != NULL )
					{
						if((pos.m_pCharkey != NULL) || (pos.m_numberKey != -1))
						{
							keyChar = this->GetKey(pData,&keyNum);
							if(keyChar == NULL)
							{
								if (  (pos.m_numberKey != -1) && (keyNum == pos.m_numberKey)  )
								{
									data = GetData( (char*)pData );
									return true;
								}
							}
							else
							{
								if ( (pos.m_pCharkey != NULL) && (pafee_strcmp(keyChar, pos.m_pCharkey ) == 0) )
								{
									data = GetData( (char*)pData );
									return true;
								}
							}
						}
						else
						{
							data = GetData( (char*)pData );
							return true;
						}
					}
				}
				pos.pos = m_pHashTable[j+1].GetFrontPosition();
				pos.m_positionIndex++;
			}
			//not found
			return false;
		}

		void Remove(
			const char* szKey///< [in] Key
		)
		{
			char* keyChar;
			int keyNum;
			unsigned int index = Hash( szKey );
			char* pData;
			int nextLength;
			List::POSITION prepos;
			List::POSITION pos = m_pHashTable[index].GetFrontPosition();
			while(pos)
			{
				prepos = pos;
				pData = (char*)m_pHashTable[index].GetNext(pos,&nextLength);
				keyChar = this->GetKey(pData,&keyNum);
				if(keyChar != NULL)
				{
					if ( pafee_strcmp(keyChar, szKey ) == 0 )
					{
						if ( m_pHashTable[index].GetCount() >=2 )
						{
							STATISTICS_DEC(m_Statistics.collisionCount);
						}
						m_pHashTable[index].RemoveAt(prepos);
						m_size--;
						STATISTICS_INC(m_Statistics.deleteCount);
						STATISTICS_DEC(m_Statistics.presentElementCount);
					}
				}
			}
			return;
		}

		void Remove(
			int key ///< [in] Key
		)
		{
			char* keyChar;
			int keyNum;
			unsigned int index = Hash( key );
			char* pData;
			int nextLength;
			List::POSITION prepos;
			List::POSITION pos = m_pHashTable[index].GetFrontPosition();
			while(pos)
			{
				prepos = pos;
				pData = (char*)m_pHashTable[index].GetNext(pos,&nextLength);
				keyChar = this->GetKey(pData,&keyNum);
				if(keyChar == NULL)
				{
					if (  keyNum == key  )
					{
						if ( m_pHashTable[index].GetCount() >=2 )
						{
							STATISTICS_DEC(m_Statistics.collisionCount);
						}
						m_pHashTable[index].RemoveAt(prepos);
						m_size--;
						STATISTICS_INC(m_Statistics.deleteCount);
						STATISTICS_DEC(m_Statistics.presentElementCount);
					}
				}
			}
			return;
		}

		void RemoveAll(void)
		{
			for(int i=0;i<m_hashSize;i++)
			{
#ifdef PAFEE_STATISTICS_ENABLE
				m_Statistics.deleteCount+=m_pHashTable[i].GetCount();
				m_Statistics.presentElementCount-=m_pHashTable[i].GetCount();
#endif
				m_pHashTable[i].RemoveAll();
			}
#ifdef PAFEE_STATISTICS_ENABLE
			m_Statistics.collisionCount=0;
#endif
			m_size=0;
			return;
		}

#ifdef PAFEE_STATISTICS_ENABLE
		void GetStatistics(STATISTICS* pStatistics)
		{
			POSITION pos = GetStartPosition();
			if(pos.pos == NULL)
			{
				m_Statistics.minListCount = 0;
				m_Statistics.maxListCount = 0;
			}
			else
			{
				for(int j=pos.m_positionIndex; j<m_hashSize; j++)
				{
					int listElementCount = m_pHashTable[j].GetCount();
					if(j == pos.m_positionIndex)
					{
						m_Statistics.minListCount = listElementCount;
						m_Statistics.maxListCount = listElementCount;
					}
					else
					{
						//min
						if(listElementCount < m_Statistics.minListCount)
						{
							m_Statistics.minListCount = listElementCount;
						}
						//max
						if(m_Statistics.maxListCount < listElementCount)
						{
							m_Statistics.maxListCount = listElementCount;
						}
					}
				}
			}
			pafee_memcpy ( pStatistics, &m_Statistics, sizeof (STATISTICS) );
		}
#endif

	private:
		int m_indexCount;
		List* m_pHashTable;
		int m_hashSize;
		int m_size;
#ifdef PAFEE_UNITTEST
	public:
#endif
		unsigned int Hash( const char *pHashKey )
		{
			unsigned int ikey=0;
			for (; *(pHashKey+3)!='\0';)
			{
				ikey ^= *(unsigned int*)pHashKey;
				(unsigned int*)pHashKey++;
			}
			return ikey%m_hashSize;
		}

		unsigned int Hash( int hashKey )
		{
			unsigned int hHight  = hashKey ^ 0xFFFF0000;
			unsigned int hLow    = hashKey ^ 0x0000FFFF;
			unsigned int hLow2   = (hHight >> 16) ^ 0x0000FFFF;
			unsigned int hHight2 = (hLow << 16)   ^ 0xFFFF0000;	
			unsigned int uhashKey = hHight2 | hLow2;
			unsigned int rhashKey = uhashKey ^ hashKey;	
			return (rhashKey%m_hashSize) & 0x7FFFFFFF;
		}

	private:
		char* BufferAllocate( int size )
		{
			char* pNewBuffer;
			pNewBuffer = (char* )Allocate(AligmentSize(size));
			return pNewBuffer;
		}

		char* GetKey( char* pData, int* pKeyNum )
		{
			short start1,start3;
			char usedCharNum=0;
			char usedCharNum1=0;
			char usedCharNum2=0;
			int keyLen;
			char* p;
			start1 = ( *(unsigned short*)pData & 0x7C00 ) >>10;
			start3 = *(short*)pData & 0x001F;
			//Character Key
			if ( ( *pData & 0x80 ) == 0x80 )
			{
				//key length
				usedCharNum = start1 - 2;
				p = pData;
				p+=2;
				if(usedCharNum == 1)
				{
					keyLen = *p & 0xFF;
				}
				else if(usedCharNum == 2)
				{
					keyLen = *(short* )p & 0xFFFF;
				}
				else
				{
					keyLen = *(int* )p & 0xFFFFFFFF;
				}
				//data length
				usedCharNum1 = start3 - start1;
				p = pData;
				p+=2+usedCharNum+usedCharNum1;
				*pKeyNum = -1;
				return p;
			}
			else
			{
				//data length
				usedCharNum1 = start1 - 2;
				//key
				usedCharNum2 = start3 - start1;
				p = pData;
				p+=2+usedCharNum1;
				if(usedCharNum2 == 1)
				{
					keyLen = *p & 0xFF;
				}
				else if(usedCharNum2 == 2)
				{
					keyLen = *(short* )p & 0xFFFF;
				}
				else
				{
					keyLen = *(int* )p & 0xFFFFFFFF;
				}
				*pKeyNum = keyLen;
				return NULL;
			}
		}

		T& GetData( char* pIn )
		{
			char* p;
			int lengthToData;
			GetDataSub(pIn, &lengthToData);
			//data
			p = pIn;
			p+=lengthToData;
			return *(T*)p;
		}

		void GetDataSub( const char* pBuffer, int* pLengthToData )
		{
			short start1,start3;
			char usedCharNum=0;
			char usedCharNum1=0;
			char usedCharNum2=0;
			const char* p;
			int keyLen;
			int dataLen = sizeof(T);
			//get data length start position
			start1 = ( *(unsigned short*)pBuffer & 0x7C00 ) >>10;
			//get data start position
			start3 = *(short*)pBuffer & 0x001F;
			//Character Key
			if ( ( *pBuffer & 0x80 ) == 0x80 )
			{
				//key length
				usedCharNum = start1 - 2;
				p = pBuffer;
				p+=2;
				if(usedCharNum == 1)
				{
					keyLen = *p & 0xFF;
				}
				else if(usedCharNum == 2)
				{
					keyLen = *(short* )p & 0xFFFF;
				}
				else
				{
					keyLen = *(int* )p & 0xFFFFFFFF;
				}
				//data length
				usedCharNum1 = start3 - start1;
				p = pBuffer;
				p+=2+usedCharNum;
				if(usedCharNum1 == 1)
				{
					dataLen = *p & 0xFF;
				}
				else if(usedCharNum1 == 2)
				{
					dataLen = *(short* )p & 0xFFFF;
				}
				else
				{
					dataLen = *(int* )p & 0xFFFFFFFF;
				}
				//data
				*pLengthToData = 2+usedCharNum+usedCharNum1+keyLen;
			}
			else
			{
				//data length
				usedCharNum1 = start1 - 2;
				p = pBuffer;
				p+=2;
				if(usedCharNum1 == 1)
				{
					dataLen = *p & 0xFF;
				}
				else if(usedCharNum1 == 2)
				{
					dataLen = *(short* )p & 0xFFFF;
				}
				else
				{
					dataLen = *(int* )p & 0xFFFFFFFF;
				}
				//key
				usedCharNum2 = start3 - start1;
				//data
				*pLengthToData = 2+usedCharNum1+usedCharNum2;
			}
		}

		//Character Key
		char* SetData(const char* szKey, const T& data, int* pBufferSize)
		{
			char* p;
			char usedCharNum=0;
			char usedCharNum1=0;
			unsigned int usedCharNum2=0;
			char* pNewBuffer;
			int size;
			int length = sizeof(data);
			//
			GetUsedCharSize(szKey,length,&usedCharNum,&usedCharNum1,&usedCharNum2);
			//create buffer
			*pBufferSize = 2+usedCharNum+usedCharNum1+usedCharNum2+length;
			pNewBuffer = BufferAllocate( *pBufferSize );
			pafee_memset(pNewBuffer,'\0',2+usedCharNum+usedCharNum1+usedCharNum2+length);
			size = SetDataSub(pNewBuffer, szKey, length, usedCharNum, usedCharNum1, usedCharNum2 );
			//data set
			p = pNewBuffer;
			p+=size;
			new (p)T;
			*(T*)p = data;
			return pNewBuffer;
		}

		void GetUsedCharSize(
			const char* szKey,
			int length,
			char* usedCharNum,
			char* usedCharNum1,
			unsigned int* usedCharNum2
		)
		{
			int maskBit;
			int usedBitCount=0;
			int usedBitCount1=0;
			//mask bit
			maskBit = 0xFFFFFFFF;
			//key length
			*usedCharNum2 = pafee_strlen(szKey)+1;
			//key used length
			while(1)
			{
				usedBitCount++;
				maskBit=maskBit<<1;
				if((*usedCharNum2 & maskBit) == 0)
				{
					break;
				}
			}
			if(usedBitCount <= 8)
			{
				*usedCharNum = 1;
			}
			else if(usedBitCount <= 16)
			{
				*usedCharNum = 2;
			}
			else
			{
				*usedCharNum = 4;
			}
			//data length
			maskBit = 0xFFFFFFFF;
			while(1)
			{
				usedBitCount1++;
				maskBit=maskBit<<1;
				if((length & maskBit) == 0)
				{
					break;
				}
			}
			if(usedBitCount1 <= 8)
			{
				*usedCharNum1 = 1;
			}
			else if(usedBitCount1 <= 16)
			{
				*usedCharNum1 = 2;
			}
			else
			{
				*usedCharNum1 = 4;
			}
		}

		int SetDataSub(
			char* pBuffer,
			const char* szKey,
			int length,
			char usedCharNum,
			char usedCharNum1,
			unsigned int usedCharNum2
		)
		{
			short startPosShort;
			char* p;
			//key type set
			*pBuffer = *pBuffer | 0x80;
			//data length start position set
			startPosShort = 2+usedCharNum;
			startPosShort<<=10;
			*(unsigned short* )pBuffer = *(unsigned short* )pBuffer | startPosShort;
			//key start position
			startPosShort = 2+usedCharNum+usedCharNum1;
			*(short* )pBuffer = *(short* )pBuffer | startPosShort;
			//key length set
			p = pBuffer;
			p+=2;
			if(usedCharNum == 1)
			{
				*p = *p | usedCharNum2;
			}
			else if(usedCharNum == 2)
			{
				*(short* )p = *(short* )p | usedCharNum2;
			}
			else
			{
				*(int* )p = *(int* )p | usedCharNum2;
			}
			//data length set
			p = pBuffer;
			p+=2+usedCharNum;
			if(usedCharNum1 == 1)
			{
				*p = *p | length;
			}
			else if(usedCharNum1 == 2)
			{
				*(short* )p = *(short* )p | length;
			}
			else
			{
				*(int* )p = *(int* )p | length;
			}
			//key set
			p = pBuffer;
			p+=2+usedCharNum+usedCharNum1;
			pafee_memcpy(p,szKey,usedCharNum2);
			//data set
			return 2+usedCharNum+usedCharNum1+usedCharNum2;
		}

		//Number Key
		char* SetData(const int key, const T& data, int* pBufferSize)
		{
			char* p;
			char usedCharNum1=0;
			char usedCharNum2=0;
			char* pNewBuffer;
			int size;
			int length = sizeof(data);
			GetUsedCharSize(key, length, &usedCharNum1, &usedCharNum2);
			//create buffer
			*pBufferSize = 2+usedCharNum1+usedCharNum2+length;
			pNewBuffer = BufferAllocate( *pBufferSize );
			pafee_memset(pNewBuffer,'\0',2+usedCharNum1+usedCharNum2+length);
			size =  SetDataSub(pNewBuffer,key,length,usedCharNum1,usedCharNum2);
			//data set
			p = pNewBuffer;
			p+=size;
			new (p)T;
			*(T*)p = data;
			return pNewBuffer;
		}

		void GetUsedCharSize( const int key, int length, char* usedCharNum1, char* usedCharNum2)
		{
			int maskBit;
			int usedBitCount1=0;
			int usedBitCount2=0;
			//mask bit
			maskBit = 0xFFFFFFFF;
			//data length
			while(1)
			{
				usedBitCount1++;
				maskBit=maskBit<<1;
				if((length & maskBit) == 0)
				{
					break;
				}
			}
			if(usedBitCount1 <= 8)
			{
				*usedCharNum1 = 1;
			}
			else if(usedBitCount1 <= 16)
			{
				*usedCharNum1 = 2;
			}
			else
			{
				*usedCharNum1 = 4;
			}
			//key length
			maskBit = 0xFFFFFFFF;
			while(1)
			{
				usedBitCount2++;
				maskBit=maskBit<<1;
				if((key & maskBit) == 0)
				{
					break;
				}
			}
			if(usedBitCount2 <= 8)
			{
				*usedCharNum2 = 1;
			}
			else if(usedBitCount2 <= 16)
			{
				*usedCharNum2 = 2;
			}
			else
			{
				*usedCharNum2 = 4;
			}
		}

		int SetDataSub(
			char* pBuffer,
			const int key,
			int length,
			char usedCharNum1,
			char usedCharNum2
		)
		{
			short startPosShort;
			char* p;
			//key type set
			*pBuffer = *pBuffer | 0x00;
			//key start position set
			startPosShort = 2+usedCharNum1;
			startPosShort<<=10;
			*(unsigned short* )pBuffer = *(unsigned short* )pBuffer | startPosShort;
			//data start position set
			startPosShort = 2+usedCharNum1+usedCharNum2;
			*(short* )pBuffer = *(short* )pBuffer | startPosShort;
			//data length set
			p = pBuffer;
			p+=2;
			if(usedCharNum1 == 1)
			{
				*p = *p | length;
			}
			else if(usedCharNum1 == 2)
			{
				*(short* )p = *(short* )p | length;
			}
			else
			{
				*(int* )p = *(int* )p | length;
			}
			//key set
			p = pBuffer;
			p+=2+usedCharNum1;
			if(usedCharNum2 == 1)
			{
				*p = *p | key;
			}
			else if(usedCharNum2 == 2)
			{
				*(short* )p = *(short* )p | key;
			}
			else
			{
				*(int* )p = *(int* )p | key;
			}
			//data start pointer
			return 2+usedCharNum1+usedCharNum2;
		}

	private:
		int m_writeIndex;
		char* m_pWriteBuffer;
		int m_writeBufferSize;
		inline int AligmentSize(int size) {return ((((size)-1)|(((size)-1)^3))+1);};
	};

}
#endif
