//****************************************************
//
//    project   Pafee
//!   @file:    PafeePriorityQueue.cpp
//!   @author:  M.Mori
//!   @brief:   PafeePriorityQueuec Class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifdef ENV_WIN32
#pragma warning(disable:4996)
#endif
#include "PafeePriorityQueue.h"
#ifdef PAFEE_UNITTEST
#include "pafee_stdio.h"
#include "pafee_stdlib.h"
#include "pafee_string.h"
#endif

namespace Pafee
{

//----------------------------------------
//! Default Constructor
//----------------------------------------
PriorityQueue::PriorityQueue(
	int priorityRange, ///< [in] priorityRange
	Allocator* pAllocator ///< [in] Pointer of allocator
)
{
	Initialize(priorityRange, pAllocator);
}

//----------------------------------------
//! Copy Constructor
//----------------------------------------
PriorityQueue::PriorityQueue(
	const PriorityQueue& pquec, ///< [in] initial value of Queue
	Allocator* pAllocator ///< [in] Memory Allocator
)
{
	Initialize(pquec.m_priorityRange, pAllocator);
	int i;
	for(i=0;i<pquec.m_priorityRange;i++)
	{
		m_pPriorityTable[i] = pquec.m_pPriorityTable[i];
	}
}

//----------------------------------------
//! Destructor
//----------------------------------------
PriorityQueue::~PriorityQueue()
{
	if ( m_pPriorityTable != NULL )
	{
		for(int i=0; i<m_priorityRange; i++)
		{
			m_pPriorityTable[i].~List();
		}
		Deallocate(m_pPriorityTable);
	}
}

//-------------------------------------------------------------------------------------
//! Copy Operator
//! @return Result(the address of Queue)
//-------------------------------------------------------------------------------------
PriorityQueue& PriorityQueue::operator =(
	const PriorityQueue& pquec ///< [in] Copied PriorityQueue
)
{
	int i;
	RemoveAll();
	for(i=0; i<m_priorityRange; i++)
	{
		m_pPriorityTable[i].~List();
	}
	if ( m_pPriorityTable != NULL )
	{
		this->Deallocate(m_pPriorityTable);
		m_pPriorityTable = NULL;
	}
	Initialize(pquec.m_priorityRange, pquec.m_pAllocator);
	for(i=0;i<pquec.m_priorityRange;i++)
	{
		m_pPriorityTable[i] = pquec.m_pPriorityTable[i];
	}
	return *this;
}

//----------------------------------------
//! Initialize
//! @return: Result(ture:success false:not success)
//----------------------------------------
bool PriorityQueue::Initialize(
	int priorityRange, ///< [in] priorityRange
	Allocator* pAllocator ///< [in] Pointer of allocator
)
{
	m_priorityRange=0;
	m_pPriorityTable=NULL;
	if ( priorityRange <= 0 )
	{
		return false;
	}
	m_priorityRange = priorityRange;
	SetAllocator(pAllocator);
	m_pPriorityTable = (List*)this->Allocate(m_priorityRange*sizeof(List));
	for(int i=0; i<m_priorityRange; i++)
	{
		new (&m_pPriorityTable[i])List(pAllocator);
	}
	return true;
}

//----------------------------------------
//! GetWriteBuffer
//! @return: Result(Write Buffer Pointer)
//----------------------------------------
void* PriorityQueue::GetWriteBuffer(
	int priority,///< [in] priority
	int length   ///< [in] Data length
)
{
	if ( m_priorityRange < priority )
	{
		return NULL;
	}
	m_pWriteBuffer = (char*)m_pPriorityTable[priority].GetWriteBuffer( AligmentSize(length) );
	return (void*)m_pWriteBuffer;
}

//----------------------------------------
//! Enqueue
//! @return: Result(true:Success false:failure)
//----------------------------------------
bool PriorityQueue::Enqueue (
	int priority,///< [in] priority
	void* pData,///< [in] Data Buffer Pointer
	int length ///< [in]  Data length
)
{
	if ( m_priorityRange < priority )
	{
		return false;
	}
	m_pPriorityTable[priority].AddBack(pData,length);
	return true;
}

//----------------------------------------
//! Enqueue
//! @return: Result(true:Success false:failure)
//----------------------------------------
bool PriorityQueue::Enqueue (
	int priority,///< [in] priority
	int length   ///< [in]  Data length
)
{
	if ( m_priorityRange < priority )
	{
		return false;
	}
	m_pPriorityTable[priority].AddBack(length);
	return true;
}

//----------------------------------------
//! Dequeue
//! @return: Result(true:Success false:failure)
//----------------------------------------
bool PriorityQueue::Dequeue(
	int* pPriority,///< [out] Pointer of priority 
	void* pBuffer, ///< [out] Data Buffer Pointer
	int* pLength,  ///< [out] Data Length Pointer
	int bufSize    ///< [in] Size of demand buffer
)
{
	for(int i=0; i<m_priorityRange; i++ )
	{
		if ( m_pPriorityTable[i].GetCount() > 0 )
		{
			if ( m_pPriorityTable[i].GetFrontLength() > bufSize )
			{
				return false;
			}
			m_pPriorityTable[i].GetFront(pBuffer,pLength);
			m_pPriorityTable[i].RemoveFront();
			*pPriority =i;
			return true;
		}
	}
	return false;
}

//----------------------------------------
//! Peek
//! @return: Result(true:Success false:failure)
//----------------------------------------
bool PriorityQueue::Peek(
	int* pPriority,///< [out] Pointer of priority 
	void* pBuffer, ///< [out] Data Buffer Pointer
	int* pLength,  ///< [out] Data Length Pointer
	int bufSize    ///< [in] Size of demand buffer
)
{
	for(int i=0;i<m_priorityRange;i++)
	{
		if ( m_pPriorityTable[i].GetCount() > 0 )
		{
			if ( m_pPriorityTable[i].GetFrontLength() > bufSize )
			{
				*pPriority =-1;
				return false;
			}
			*pPriority =i;
			m_pPriorityTable[i].GetFront(pBuffer,pLength);
			return true;
		}
	}
	*pPriority =-1;
	return false;
}

//----------------------------------------
//! Peek
//! @return: Result(Peek Buffer Pointer)
//----------------------------------------
void* PriorityQueue::Peek (
	int* pPriority,///< [out] Pointer of priority 
	int* pLength   ///< [out] Data Length Pointer
)
{
	for(int i=0;i<m_priorityRange;i++)
	{
		if ( m_pPriorityTable[i].GetCount() > 0 )
		{
			*pPriority =i;
			return m_pPriorityTable[i].GetFront(pLength);
		}
	}
	*pLength=-1;
	return NULL;
}

//----------------------------------------
//! Peek
//! @return: Result(Peek Length)
//----------------------------------------
int PriorityQueue::PeekLength(
	int* pPriority ///< [out] Pointer of priority 
)
{
	for(int i=0;i<m_priorityRange;i++)
	{
		if ( m_pPriorityTable[i].GetCount() > 0 )
		{
			*pPriority = i;
			return m_pPriorityTable[i].GetFrontLength();
		}
	}
	*pPriority = -1;
	return -1;
}

//----------------------------------------
//! RemoveFront
//! @return: void
//----------------------------------------
void PriorityQueue::RemoveFront(void)
{
	for(int i=0; i<m_priorityRange; i++ )
	{
		if (m_pPriorityTable[i].GetCount()!=0)
		{
			m_pPriorityTable[i].RemoveFront();
			return;
		}
	}
	return;
}

//----------------------------------------
//! RemoveAll
//! @return: void
//----------------------------------------
void PriorityQueue::RemoveAll(void)
{
	for(int i=0; i<m_priorityRange; i++ )
	{
		int cMax = m_pPriorityTable[i].GetCount();
		for(int j=0; j<cMax; j++ )
		{
			if ( m_pPriorityTable[i].GetCount() == 0 )
			{
				break;
			}
			m_pPriorityTable[i].RemoveFront();
		}
	}
	return;
}

//----------------------------------------
//! GetStatistics
//! @return: void
//----------------------------------------
#ifdef PAFEE_STATISTICS_ENABLE
void PriorityQueue::GetStatistics(PriorityQueue::STATISTICS* pStatistics)
{
		Pafee::List::STATISTICS statisticsRtn;
		m_Statistics.addCount = 0;
		m_Statistics.maxElementCount = 0;
		m_Statistics.maxElementSize = 0;
		m_Statistics.minElementSize = 0;
		m_Statistics.presentElementCount = 0;
		List *p=m_pPriorityTable;
		for(int i=0;i<m_priorityRange;i++)
		{
			p->GetStatistics(&statisticsRtn);
			m_Statistics.addCount += statisticsRtn.addCount;
			m_Statistics.maxElementCount += statisticsRtn.maxElementCount;
			if ( i== 0 )
			{
				m_Statistics.maxElementSize = statisticsRtn.maxElementSize;
				m_Statistics.minElementSize = statisticsRtn.minElementSize;
			}
			if (m_Statistics.maxElementSize < statisticsRtn.maxElementSize)
			{
				m_Statistics.maxElementSize = statisticsRtn.maxElementSize;
			}
			if (m_Statistics.minElementSize > statisticsRtn.minElementSize)
			{
				m_Statistics.minElementSize = statisticsRtn.minElementSize;
			}
			m_Statistics.presentElementCount += statisticsRtn.presentElementCount;
			p++;
		}
		pafee_memcpy ( pStatistics, &m_Statistics, sizeof (PriorityQueue::STATISTICS) );
}
#endif

//----------------------------------------------------------------------------
//!	Unit test
//! @return: void
//----------------------------------------------------------------------------
#ifdef PAFEE_UNITTEST
void PriorityQueue::UnitTest(void)
{
	int i;
	int getdata;
	int length,pr;
	Pafee::PriorityQueue  pq;
	int size = sizeof(int);
	//STATISTICS
	int addCount=0;
	int maxElementCount=0;
	int minElementSize=0;
	int maxElementSize=0;
	int presentElementCount=0;

#ifdef PAFEE_STATISTICS_ENABLE
	PriorityQueue::STATISTICS statisticsRtn;
#endif
	//Normality system
	assert(pq.GetCount() == 0);
	assert(pq.IsEmpty());
	assert(pq.Initialize(3));
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == minElementSize);
	assert(statisticsRtn.maxElementSize == maxElementSize);
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	int Counti=0;
	//Enqueue Test
	for(i=0;i<12;i++)
	{
		assert(pq.GetCount() == i);
		assert(pq.Enqueue (i%3, &i, sizeof(int)));
		assert(pq.GetCount() == i+1);
		assert(!pq.IsEmpty());
		addCount++;
		presentElementCount++;
	}
	maxElementCount = 12;
	minElementSize = size;
	maxElementSize = size;
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == minElementSize);
	assert(statisticsRtn.maxElementSize == maxElementSize);
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	Counti = i;
	for(i=0;i<4;i++)
	{
		assert( pq.Peek(&pr, &getdata, &length, sizeof(int)-1)==false);
		assert( pq.Peek(&pr, &getdata, &length, sizeof(int)));
		assert( pq.PeekLength(&pr) == length);
		assert(pq.GetCount() == Counti);
		assert(getdata==i*3);
		assert(pr==0);
		assert(length==sizeof(int));
		assert(pq.Dequeue (&pr, &getdata, &length, sizeof(int)-1)==false);
		assert(pq.Dequeue (&pr, &getdata, &length, sizeof(int)));
		Counti--;
		assert(pq.GetCount() == Counti);
		assert(getdata==i*3);
		assert(pr==0);
		assert(length==sizeof(int));
		presentElementCount--;
	}
	for(i=0;i<4;i++)
	{
		assert( pq.Peek(&pr, &getdata, &length, sizeof(int)-1)==false);
		assert( pq.Peek(&pr, &getdata, &length, sizeof(int)));
		assert( pq.PeekLength(&pr) == length);
		assert(pq.GetCount() == Counti);
		assert(getdata==i*3+pr);
		assert(pr==1);
		assert(length==sizeof(int));
		assert(pq.Dequeue (&pr, &getdata, &length, sizeof(int)-1)==false);
		assert(pq.Dequeue (&pr, &getdata, &length, sizeof(int)));
		Counti--;
		assert(pq.GetCount() == Counti);
		assert(getdata==i*3+pr);
		assert(pr==1);
		assert(length==sizeof(int));
		presentElementCount--;
	}
	for(i=0;i<4;i++)
	{
		assert( pq.Peek(&pr, &getdata, &length, sizeof(int)-1)==false);
		assert( pq.Peek(&pr, &getdata, &length, sizeof(int)));
		assert( pq.PeekLength(&pr) == length);
		assert(pq.GetCount() == Counti);
		assert(getdata==i*3+pr);
		assert(pr==2);
		assert(length==sizeof(int));
		assert(pq.Dequeue (&pr, &getdata, &length, sizeof(int)-1)==false);
		assert(pq.Dequeue (&pr, &getdata, &length, sizeof(int)));
		Counti--;
		assert(pq.GetCount() == Counti);
		assert(getdata==i*3+pr);
		assert(pr==2);
		assert(length==sizeof(int));
		presentElementCount--;
	}
	assert(pq.GetCount() == 0);
	assert(!pq.Dequeue (&pr, &getdata, &length, sizeof(int)));
	maxElementCount = 12;
	minElementSize = 0;
	maxElementSize = 0;

#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == minElementSize);
	assert(statisticsRtn.maxElementSize == maxElementSize);
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	//RemoveFront Test
	Counti = 0;
	for(i=0;i<12;i++)
	{
		assert(pq.Enqueue (i%3, &i, sizeof(int)));
		presentElementCount++;
		addCount++;
		Counti++;		
	}
	for(i=0;i<12;i++)
	{
		pq.RemoveFront();
		presentElementCount--;
		Counti--;
		assert(pq.GetCount() == Counti);
	}
	//RemoveALL Test
	for(i=0;i<12;i++)
	{
		assert(pq.GetCount() == i);
		assert(pq.Enqueue (i%3, &i, sizeof(int)));
		addCount++;
		presentElementCount++;
		assert(pq.GetCount() == i+1);
		assert(!pq.IsEmpty());	
	}
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == sizeof(int));
	assert(statisticsRtn.maxElementSize == sizeof(int));
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	pq.RemoveAll();
	presentElementCount=0;
	assert(pq.GetCount() == 0);
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == 0);
	assert(statisticsRtn.maxElementSize == 0);
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	char *pWriteBuffer;
	//Light Version
	static char databuffer[12][16];
	int setsize;
//	int ad = addCount;
	int minsetsize = 0;
	for(i=0;i<12;i++)
	{
		setsize = pafee_sprintf(databuffer[i],"ABC%d",i);
		if ( i==0 )
		{
			minsetsize = setsize;
		}
		assert(pq.GetCount() == i);
		pWriteBuffer = (char*)pq.GetWriteBuffer(i%3, 16);
		assert(pWriteBuffer != NULL );
		pafee_strcpy ( pWriteBuffer, databuffer[i]);
		assert(pq.Enqueue (i%3, setsize));
		addCount++;
		Counti++;
		presentElementCount++;
		assert(pq.GetCount() == Counti);

	}
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == minsetsize);
	assert(statisticsRtn.maxElementSize == setsize);
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	Counti=12;
	char* pData;
	char getdatastr[16];
	for(i=0;i<4;i++)
	{
		pData = (char*)pq.Peek(&pr,&length);
		assert( pq.PeekLength(&pr) == length);
		assert(pq.GetCount() == Counti);
		assert(strcmp(pData,databuffer[i*3+pr])==0);
		assert(pr==0);
		assert((unsigned int)length==strlen(databuffer[i*3+pr]));
		assert(pq.Dequeue (&pr, getdatastr, &length, 1)==false);
		assert(pq.Dequeue (&pr, getdatastr, &length, 16));
		Counti--;
		assert(pq.GetCount() == Counti);
		getdatastr[length]='\0';
		assert(strcmp(getdatastr,databuffer[i*3+pr])==0);
		assert(pr==0);
		assert((unsigned int)length==strlen(databuffer[i*3+pr]));
	}
	for(i=0;i<4;i++)
	{
		pData = (char*)pq.Peek(&pr,&length);
		assert( pq.PeekLength(&pr) == length);
		assert(pq.GetCount() == Counti);
		assert(strcmp(pData,databuffer[i*3+pr])==0);
		assert(pr==1);
		assert((unsigned int)length==strlen(databuffer[i*3+pr]));
		assert(pq.Dequeue (&pr, getdatastr, &length, 1)==false);
		assert(pq.Dequeue (&pr, getdatastr, &length, 16));
		Counti--;
		assert(pq.GetCount() == Counti);
		getdatastr[length]='\0';
		assert(strcmp(getdatastr,databuffer[i*3+pr])==0);
		assert(pr==1);
		assert((unsigned int)length==strlen(databuffer[i*3+pr]));
	}
	for(i=0;i<4;i++)
	{
		pData = (char*)pq.Peek(&pr,&length);
		assert( pq.PeekLength(&pr) == length);
		assert(pq.GetCount() == Counti);
		assert(strcmp(pData,databuffer[i*3+pr])==0);
		assert(pr==2);
		assert((unsigned int)length==strlen(databuffer[i*3+pr]));
		assert(pq.Dequeue (&pr, getdatastr, &length, 1)==false);
		assert(pq.Dequeue (&pr, getdatastr, &length, 16));
		Counti--;
		assert(pq.GetCount() == Counti);
		getdatastr[length]='\0';
		assert(strcmp(getdatastr,databuffer[i*3+pr])==0);
		assert(pr==2);
		assert((unsigned int)length==strlen(databuffer[i*3+pr]));
	}
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == 0);
	assert(statisticsRtn.maxElementSize == 0);
	assert(statisticsRtn.presentElementCount == 0);
#endif
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
	maxElementCount=0;
	minElementSize=0;
	maxElementSize=0;
	presentElementCount=0;
	Counti=0;
	//Enqueue Test
	for(i=0;i<12;i++)
	{
		assert(pq.GetCount() == i);
		assert(pq.Enqueue (i%3, &i, sizeof(int)));
		assert(pq.GetCount() == i+1);
		assert(!pq.IsEmpty());
		addCount++;
		presentElementCount++;
	}
	maxElementCount = 12;
	minElementSize = size;
	maxElementSize = size;
#ifdef PAFEE_STATISTICS_ENABLE
	pq.GetStatistics(&statisticsRtn);
	assert(statisticsRtn.addCount == addCount);
	assert(statisticsRtn.maxElementCount == maxElementCount);
	assert(statisticsRtn.minElementSize == minElementSize);
	assert(statisticsRtn.maxElementSize == maxElementSize);
	assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
	Pafee::PriorityQueue  pq2;
	pq2=pq;
	int addCount2=addCount;
	int maxElementCount2=maxElementCount;
	int minElementSize2=minElementSize;
	int maxElementSize2=maxElementSize;
	int presentElementCount2=presentElementCount;
#ifdef PAFEE_STATISTICS_ENABLE
	PriorityQueue::STATISTICS statisticsRtn2;
#endif
	int Counti2 = i;
	for(i=0;i<4;i++)
	{
		assert( pq2.Peek(&pr, &getdata, &length, sizeof(int)-1)==false);
		assert( pq2.Peek(&pr, &getdata, &length, sizeof(int)));
		assert( pq2.PeekLength(&pr) == length);
		assert(pq2.GetCount() == Counti2);
		assert(getdata==i*3);
		assert(pr==0);
		assert(length==sizeof(int));
		assert(pq2.Dequeue (&pr, &getdata, &length, sizeof(int)-1)==false);
		assert(pq2.Dequeue (&pr, &getdata, &length, sizeof(int)));
		Counti2--;
		assert(pq2.GetCount() == Counti2);
		assert(getdata==i*3);
		assert(pr==0);
		assert(length==sizeof(int));
		presentElementCount2--;
	}
	for(i=0;i<4;i++)
	{
		assert( pq2.Peek(&pr, &getdata, &length, sizeof(int)-1)==false);
		assert( pq2.Peek(&pr, &getdata, &length, sizeof(int)));
		assert( pq2.PeekLength(&pr) == length);
		assert(pq2.GetCount() == Counti2);
		assert(getdata==i*3+pr);
		assert(pr==1);
		assert(length==sizeof(int));
		assert(pq2.Dequeue (&pr, &getdata, &length, sizeof(int)-1)==false);
		assert(pq2.Dequeue (&pr, &getdata, &length, sizeof(int)));
		Counti2--;
		assert(pq2.GetCount() == Counti2);
		assert(getdata==i*3+pr);
		assert(pr==1);
		assert(length==sizeof(int));
		presentElementCount2--;
	}
	for(i=0;i<4;i++)
	{
		assert( pq2.Peek(&pr, &getdata, &length, sizeof(int)-1)==false);
		assert( pq2.Peek(&pr, &getdata, &length, sizeof(int)));
		assert( pq2.PeekLength(&pr) == length);
		assert(pq2.GetCount() == Counti2);
		assert(getdata==i*3+pr);
		assert(pr==2);
		assert(length==sizeof(int));
		assert(pq2.Dequeue (&pr, &getdata, &length, sizeof(int)-1)==false);
		assert(pq2.Dequeue (&pr, &getdata, &length, sizeof(int)));
		Counti2--;
		assert(pq2.GetCount() == Counti2);
		assert(getdata==i*3+pr);
		assert(pr==2);
		assert(length==sizeof(int));
		presentElementCount2--;
	}
	assert(pq2.GetCount() == 0);
	assert(!pq2.Dequeue (&pr, &getdata, &length, sizeof(int)));
	maxElementCount = 12;
	minElementSize2 = 0;
	maxElementSize2 = 0;
#ifdef PAFEE_STATISTICS_ENABLE
	pq2.GetStatistics(&statisticsRtn2);
	assert(statisticsRtn2.addCount == addCount2);
	assert(statisticsRtn2.maxElementCount == maxElementCount2);
	assert(statisticsRtn2.minElementSize == minElementSize2);
	assert(statisticsRtn2.maxElementSize == maxElementSize2);
	assert(statisticsRtn2.presentElementCount == presentElementCount2);
#endif
	//RemoveFront Test
	Counti2 = 0;
	for(i=0;i<12;i++)
	{
		assert(pq2.Enqueue (i%3, &i, sizeof(int)));
		presentElementCount2++;
		addCount2++;
		Counti2++;		
	}
	for(i=0;i<12;i++)
	{
		pq2.RemoveFront();
		presentElementCount2--;
		Counti2--;
		assert(pq2.GetCount() == Counti2);
	}
	//RemoveALL Test
	for(i=0;i<12;i++)
	{
		assert(pq2.GetCount() == i);
		assert(pq2.Enqueue (i%3, &i, sizeof(int)));
		addCount2++;
		presentElementCount2++;
		assert(pq2.GetCount() == i+1);
		assert(!pq2.IsEmpty());	
	}
#ifdef PAFEE_STATISTICS_ENABLE
	pq2.GetStatistics(&statisticsRtn2);
	assert(statisticsRtn2.addCount == addCount2);
	assert(statisticsRtn2.maxElementCount == maxElementCount);
	assert(statisticsRtn2.minElementSize == sizeof(int));
	assert(statisticsRtn2.maxElementSize == sizeof(int));
	assert(statisticsRtn2.presentElementCount == presentElementCount2);
#endif
	pq2.RemoveAll();
	presentElementCount2=0;
	assert(pq2.GetCount() == 0);
#ifdef PAFEE_STATISTICS_ENABLE
	pq2.GetStatistics(&statisticsRtn2);
	assert(statisticsRtn2.addCount == addCount2);
	assert(statisticsRtn2.maxElementCount == maxElementCount);
	assert(statisticsRtn2.minElementSize == 0);
	assert(statisticsRtn2.maxElementSize == 0);
	assert(statisticsRtn2.presentElementCount == presentElementCount2);
#endif
	char *pWriteBuffer2;
	//Light Version
	static char databuffer2[12][16];
	int setsize2;
	int minsetsize2 = 0;
	for(i=0;i<12;i++)
	{
		setsize2 = pafee_sprintf(databuffer2[i],"ABC%d",i);
		if ( i==0 )
		{
			minsetsize2 = setsize2;
		}
		assert(pq2.GetCount() == i);
		pWriteBuffer2 = (char*)pq2.GetWriteBuffer(i%3, 16);
		assert(pWriteBuffer2 != NULL );
		pafee_strcpy ( pWriteBuffer2, databuffer2[i]);
		assert(pq2.Enqueue (i%3, setsize2));
		addCount2++;
		Counti2++;
		presentElementCount2++;
		assert(pq2.GetCount() == Counti2);
	}
#ifdef PAFEE_STATISTICS_ENABLE
	pq2.GetStatistics(&statisticsRtn2);
	assert(statisticsRtn2.addCount == addCount2);
	assert(statisticsRtn2.maxElementCount == maxElementCount);
	assert(statisticsRtn2.minElementSize == minsetsize2);
	assert(statisticsRtn2.maxElementSize == setsize2);
	assert(statisticsRtn2.presentElementCount == presentElementCount2);
#endif
	return;
	/*
	Counti2=12;
	char* pData2;
	char getdatastr2[16];
	for(i=0;i<4;i++)
	{
		pData2 = (char*)pq2.Peek(&pr,&length);
		assert( pq2.PeekLength(&pr) == length);
		assert(pq2.GetCount() == Counti2);
		assert(strcmp(pData2,databuffer2[i*3+pr])==0);
		assert(pr==0);
		assert((unsigned int)length==strlen(databuffer2[i*3+pr]));
		assert(pq2.Dequeue (&pr, getdatastr2, &length, 1)==false);
		assert(pq2.Dequeue (&pr, getdatastr2, &length, 16));
		Counti2--;
		assert(pq2.GetCount() == Counti2);
		getdatastr2[length]='\0';
		assert(strcmp(getdatastr2,databuffer2[i*3+pr])==0);
		assert(pr==0);
		assert((unsigned int)length==strlen(databuffer2[i*3+pr]));
	}
	for(i=0;i<4;i++)
	{
		pData2 = (char*)pq2.Peek(&pr,&length);
		assert( pq2.PeekLength(&pr) == length);
		assert(pq2.GetCount() == Counti2);
		assert(strcmp(pData2,databuffer2[i*3+pr])==0);
		assert(pr==1);
		assert((unsigned int)length==strlen(databuffer2[i*3+pr]));
		assert(pq2.Dequeue (&pr, getdatastr2, &length, 1)==false);
		assert(pq2.Dequeue (&pr, getdatastr2, &length, 16));
		Counti2--;
		assert(pq2.GetCount() == Counti2);
		getdatastr2[length]='\0';
		assert(strcmp(getdatastr2,databuffer2[i*3+pr])==0);
		assert(pr==1);
		assert((unsigned int)length==strlen(databuffer2[i*3+pr]));
	}
	for(i=0;i<4;i++)
	{
		pData2 = (char*)pq2.Peek(&pr,&length);
		assert( pq2.PeekLength(&pr) == length);
		assert(pq2.GetCount() == Counti2);
		assert(strcmp(pData2,databuffer2[i*3+pr])==0);
		assert(pr==2);
		assert((unsigned int)length==strlen(databuffer2[i*3+pr]));
		assert(pq2.Dequeue (&pr, getdatastr2, &length, 1)==false);
		assert(pq2.Dequeue (&pr, getdatastr2, &length, 16));
		Counti2--;
		assert(pq2.GetCount() == Counti2);
		getdatastr2[length]='\0';
		assert(strcmp(getdatastr2,databuffer2[i*3+pr])==0);
		assert(pr==2);
		assert((unsigned int)length==strlen(databuffer2[i*3+pr]));
	}
#ifdef PAFEE_STATISTICS_ENABLE
	pq2.GetStatistics(&statisticsRtn2);
	assert(statisticsRtn2.addCount == addCount2);
	assert(statisticsRtn2.maxElementCount == maxElementCount);
	assert(statisticsRtn2.minElementSize == 0);
	assert(statisticsRtn2.maxElementSize == 0);
	assert(statisticsRtn2.presentElementCount == 0);
#endif
	return;
	*/
}
#endif

}
