//****************************************************
//
//      project    PAFEE
//!     @file:     PafeeRingBuffer.cpp
//!     @author:   M.Mori
//!     @brief:    RingBuffer class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "PafeeRingBuffer.h"

namespace Pafee
{

//--------------------------------------
//! Default Constructor
//--------------------------------------
RingBuffer::RingBuffer(
	int capacity, ///< [in] Maximum size and memory allocator of number of elements
	int elementSize, ///< [in] and one element in can setting of the following as argument and ring buffer
	Allocator* pAllocator ///< [in] MemoryAllocator
)
{
	Initialize(capacity, elementSize, pAllocator);
}

//--------------------------------------
//! Copy Constructor
//--------------------------------------
RingBuffer::RingBuffer(
	RingBuffer& rbc, ///< [in] initial value of RingBuffer
	Allocator* pAllocator ///< [in] MemoryAllocator
)
{
	Initialize(rbc.m_capacity, rbc.m_elementSize, pAllocator);
	pafee_memcpy(this->m_pBuffer,rbc.m_pBuffer,rbc.m_bufferAllocateLength);
	{
		for(register int i=0;i<m_capacity;i++)
		{
			m_pRingBufferPointer[i].Size=rbc.m_pRingBufferPointer[i].Size;
		}
	}
	m_pRead  = rbc.m_pRead  - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
	m_pWrite = rbc.m_pWrite - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
	m_Statistics.additionalFrequency=rbc.m_Statistics.additionalFrequency;
	m_Statistics.maximumElements=rbc.m_Statistics.maximumElements;
	m_Statistics.overFlowFrequency=rbc.m_Statistics.overFlowFrequency;
	m_Statistics.presentNumber=rbc.m_Statistics.presentNumber;
#endif
}

//--------------------------------------
//! Destructor
//-------------------------------------- 
RingBuffer::~RingBuffer()
{
	if ( m_pRingBufferPointer != NULL )
	{
		Deallocate(m_pRingBufferPointer);
	}
	if ( m_pBuffer != NULL )
	{
		Deallocate(m_pBuffer);
	}
}

//-----------------------------------------------------------------------------
//! Initialize
//! @return Result(true:Success  false: Failure)
//-----------------------------------------------------------------------------
bool RingBuffer::Initialize(
	int capacity,  ///< [in] capacity
	int elementSize,  ///< [in] elementSize
	Allocator* pAllocator ///< [in] MemoryAllocator
)
{
	SetAllocator(pAllocator);
	m_pRingBufferPointer = NULL;
	m_pBuffer = NULL;
	m_capacity = capacity;
	m_elementSize= elementSize;
	if ( (capacity == 0) || (elementSize == 0) )
	{
		return false;
	}
	m_bufferLength = capacity*m_elementSize;
	m_bufferAllocateLength = AligmentSize(m_bufferLength);
	m_pBuffer = (char*)Allocate(m_bufferAllocateLength);
	m_pRingBufferPointer = (RingBufferPointer*)Allocate(capacity*sizeof(RingBufferPointer));
	{
		register char* pBuf = m_pBuffer;
		for(register int i=0;i<m_capacity;i++)
		{
			m_pRingBufferPointer[i].p=pBuf;
			m_pRingBufferPointer[i].pNext=&m_pRingBufferPointer[i+1];
			m_pRingBufferPointer[i].Size=0;
			pBuf+=m_elementSize;
		}
		m_pRingBufferPointer[m_capacity-1].pNext=m_pRingBufferPointer;
	}
	m_pRead = m_pWrite = m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
	m_Statistics.additionalFrequency=0;
	m_Statistics.maximumElements=0;
	m_Statistics.overFlowFrequency=0;
	m_Statistics.presentNumber=0;
#endif
	return true;
}

//-------------------------------------------------------------------------------------
//! operator =
//! @return Result(the address of RingBuffer)
//-------------------------------------------------------------------------------------
RingBuffer& RingBuffer::operator =(
	RingBuffer& rbc ///< [in] initial value of RingBuffer
)
{
	if ( m_pRingBufferPointer != NULL )
	{
		Deallocate(m_pRingBufferPointer);
	}
	if ( m_pBuffer != NULL )
	{
		Deallocate(m_pBuffer);
	}
	m_pRingBufferPointer = NULL;
	m_pBuffer = NULL;
	Initialize(rbc.m_capacity, rbc.m_elementSize, m_pAllocator);
	pafee_memcpy(this->m_pBuffer,rbc.m_pBuffer,rbc.m_bufferAllocateLength);
	{
		for(register int i=0;i<m_capacity;i++)
		{
			m_pRingBufferPointer[i].Size=rbc.m_pRingBufferPointer[i].Size;
		}
	}
	m_pRead  = rbc.m_pRead  - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
	m_pWrite = rbc.m_pWrite - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
	m_Statistics.additionalFrequency=rbc.m_Statistics.additionalFrequency;
	m_Statistics.maximumElements=rbc.m_Statistics.maximumElements;
	m_Statistics.overFlowFrequency=rbc.m_Statistics.overFlowFrequency;
	m_Statistics.presentNumber=rbc.m_Statistics.presentNumber;
#endif
	return *this;
}

//-----------------------------------------------------------------------------
//!	GetWriteBuffer
//!	@return WritePointer
//-----------------------------------------------------------------------------
void* RingBuffer::GetWriteBuffer()
{
	return (void *)m_pWrite->p;
}

//-------------------------------------------------------------------------------
//! Peek
//! @return Result(true:Success  false: Failure)
//------------------------------------------------------------------------------- 
bool RingBuffer::Peek(
	void* pBuffer, ///< [out] Prepared Buffer
	int* pLength, ///< [out] Data Length
	int bufSize ///< [in] Prepared Buffer Length
)
{
	if ( m_pRead->Size == 0 )
	{
		return false;
	}
	*pLength = m_pRead->Size;
	if(bufSize < *pLength)
	{
		return false;
	}
	pafee_memcpy ( pBuffer, m_pRead->p, *pLength );
	return true;
}

//-------------------------------------------------------------------------------
//! Peek
//! @return Result(Data Pointer)
//-------------------------------------------------------------------------------
void* RingBuffer::Peek(
	int* pLength ///< [out] Data Length
)
{
	if ( m_pRead->Size == 0 )
	{
		return NULL;
	}
	*pLength = m_pRead->Size;
	return (void *)m_pRead->p;
}

//-------------------------------------------------------------------------------
//! PeekLength
//! @return Result(Length:Success  0: Failure)
//-------------------------------------------------------------------------------
int RingBuffer::PeekLength(void)
{
	if ( m_pRead->Size == 0 )
	{
		return 0;
	}
	return m_pRead->Size;
}

//-------------------------------------------------------------------------------
//! ClearAll
//! @return void
//-------------------------------------------------------------------------------
void RingBuffer::ClearAll(void)
{
	for(int i=0;i<m_capacity;i++)
	{
		m_pRingBufferPointer[i].Size=0;
	}
	m_pRead = m_pWrite = m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
	m_Statistics.presentNumber=0;
#endif
	return;
}

#ifdef PAFEE_STATISTICS_ENABLE
//-------------------------------------------------------------------------------
//! Statistical information is acquired.
//! @return void
//-------------------------------------------------------------------------------
void RingBuffer::GetStatistics(
	RingBuffer::STATISTICS* pStatistics ///< [out] Statistics Pointer
)
{
	pafee_memcpy ( pStatistics, &m_Statistics, sizeof (RingBuffer::STATISTICS) );
}
#endif

#ifdef PAFEE_UNITTEST
//----------------------------------------------------------------------------
//! Unit test
//! @return: void
//----------------------------------------------------------------------------
void RingBuffer::UnitTest(void)
{
	{
		const int TEST_RING_BUFFER_CAPACITY=10;
		const int TEST_RING_BUFFER_ELEMENTSIZE=4;
		int pData;
		int	i;
		int GetData;
		int GetLength;
		int PeekData;
		//STATISTICS
		int additionalFrequency=0;
		int maximumElements=0;
		int overFlowFrequency=0;
		int presentNumber=0;
#ifdef PAFEE_STATISTICS_ENABLE
		RingBuffer::STATISTICS statisticsRtn;
#endif// PAFEE_STATISTICS_ENABLE
		//test GetCount/IsEmpty/Peek/Enqueue/Dequeue
		{
			RingBuffer buffer(TEST_RING_BUFFER_CAPACITY, TEST_RING_BUFFER_ELEMENTSIZE);
			assert(buffer.IsEmpty() == true);
			assert(buffer.GetCount() == 0);
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.Dequeue(&GetData,&GetLength) == false);
			//Enqueue test
			for(i=0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				maximumElements++;
				presentNumber++;
			}
			assert(!buffer.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Enqueue
			assert(buffer.Enqueue(&i,sizeof(i)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			// Dequeue test
			for(i = 0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Peek(&PeekData,&GetLength));
				assert(PeekData == i);
				assert(buffer.Dequeue(&GetData,&GetLength));
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.IsEmpty());
			assert(buffer.Dequeue(&GetData,&GetLength) == false);
			// Enqueue and Dequeue test
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
			assert(!buffer.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Enqueue
			assert(buffer.Enqueue(&i,sizeof(i)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				assert(buffer.Peek(&PeekData,&GetLength));
				assert(PeekData == i);
				assert(buffer.Dequeue(&GetData,&GetLength) == true);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!buffer.IsEmpty());
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!buffer.IsEmpty());
			pData=0;
			assert(buffer.Enqueue(&pData,sizeof(pData)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Dequeue(&GetData,&GetLength) == true);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.IsEmpty());
			assert(buffer.Dequeue(&GetData,&GetLength) == false);
			//test	void ClearAll(void);
			for(i=0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			buffer.ClearAll();
			presentNumber=0;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.IsEmpty());
			assert(buffer.Dequeue(&GetData,&GetLength) == false);

			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)//1
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.Enqueue(&i,sizeof(i)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
		}
		//test pattern 2
		//Ministry copy vesion
		//test GetCount/IsEmpty/Peek/PeekLength/Enqueue/Dequeue(Ministry copy vesion)
		//
		int*   pWritePointer;
		{
			RingBuffer Buffer;
			assert(Buffer.Initialize(0,0,NULL) == false);
			assert(Buffer.Initialize(TEST_RING_BUFFER_CAPACITY, TEST_RING_BUFFER_ELEMENTSIZE,NULL) == true);
			assert(Buffer.IsEmpty() == true);
			assert(Buffer.GetCount() == 0);
			assert(Buffer.Dequeue(&GetLength) == false);
			assert(Buffer.PeekLength() == 0UL);
			additionalFrequency=0;
			maximumElements=0;
			overFlowFrequency=0;
			presentNumber=0;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			*pWritePointer = 0;
			assert(Buffer.Enqueue(sizeof(int)) == true);
			additionalFrequency++;
			maximumElements++;
			presentNumber++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			*pWritePointer = 1;
			assert(Buffer.Enqueue(sizeof(int)) == true);
			additionalFrequency++;
			maximumElements++;
			presentNumber++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			Buffer.Dequeue(&GetLength);
			presentNumber--;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			Buffer.Dequeue(&GetLength);
			presentNumber--;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Enqueue test
			for(i=0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				pWritePointer = (int *)Buffer.GetWriteBuffer();
				*pWritePointer = i;
				assert(Buffer.Enqueue(sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
			maximumElements = TEST_RING_BUFFER_CAPACITY;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!Buffer.IsEmpty());
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			assert(Buffer.Enqueue(sizeof (int)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			// Dequeue test
			for(i = 0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				PeekData = *(int *)Buffer.Peek(&GetLength);
				assert(PeekData == i);
				assert(Buffer.PeekLength() == GetLength);
				GetData = *(int *)Buffer.Dequeue(&GetLength);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(Buffer.IsEmpty());
			assert(Buffer.Dequeue(&GetLength) == false);
			// Enqueue and Dequeue test
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				pWritePointer = (int *)Buffer.GetWriteBuffer();
				*pWritePointer = i;
				assert(Buffer.Enqueue(sizeof (int)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!Buffer.IsEmpty());
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			assert(Buffer.Enqueue(sizeof (int)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				PeekData = *(int *)Buffer.Peek(&GetLength);
				assert(Buffer.PeekLength() == GetLength);
				GetData = *(int *)Buffer.Dequeue(&GetLength);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif

			assert(!Buffer.IsEmpty());
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				pWritePointer = (int *)Buffer.GetWriteBuffer();
				*pWritePointer = i;
				assert(Buffer.Enqueue(sizeof (int)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif

			assert(!Buffer.IsEmpty());
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			assert(Buffer.Enqueue(sizeof (int)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				PeekData = *(int *)Buffer.Peek(&GetLength);
				assert(Buffer.PeekLength() == GetLength);
				GetData = *(int *)Buffer.Dequeue(&GetLength);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(Buffer.IsEmpty());
			assert(Buffer.Dequeue(&GetLength) == false);
		}
	}
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////
	{
		const int TEST_RING_BUFFER_CAPACITY=10;
		const int TEST_RING_BUFFER_ELEMENTSIZE=4;
		int pData;
		int	i;
		int GetData;
		int GetLength;
		int PeekData;
		//STATISTICS
		int additionalFrequency=0;
		int maximumElements=0;
		int overFlowFrequency=0;
		int presentNumber=0;
#ifdef PAFEE_STATISTICS_ENABLE
		RingBuffer::STATISTICS statisticsRtn;
#endif// PAFEE_STATISTICS_ENABLE
		//test GetCount/IsEmpty/Peek/Enqueue/Dequeue
		{
			RingBuffer buffer0c(TEST_RING_BUFFER_CAPACITY, TEST_RING_BUFFER_ELEMENTSIZE);
			assert(buffer0c.IsEmpty() == true);
			assert(buffer0c.GetCount() == 0);
#ifdef PAFEE_STATISTICS_ENABLE
			buffer0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer0c.Dequeue(&GetData,&GetLength) == false);
			//Enqueue test
			for(i=0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer0c.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				maximumElements++;
				presentNumber++;
			}
			assert(!buffer0c.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			buffer0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Enqueue
			assert(buffer0c.Enqueue(&i,sizeof(i)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			RingBuffer buffer=buffer0c;

			// Dequeue test
			for(i = 0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Peek(&PeekData,&GetLength));
				assert(PeekData == i);
				assert(buffer.Dequeue(&GetData,&GetLength));
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.IsEmpty());
			assert(buffer.Dequeue(&GetData,&GetLength) == false);
			// Enqueue and Dequeue test
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
			assert(!buffer.IsEmpty());
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Enqueue
			assert(buffer.Enqueue(&i,sizeof(i)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				assert(buffer.Peek(&PeekData,&GetLength));
				assert(PeekData == i);
				assert(buffer.Dequeue(&GetData,&GetLength) == true);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!buffer.IsEmpty());
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!buffer.IsEmpty());
			pData=0;
			assert(buffer.Enqueue(&pData,sizeof(pData)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Dequeue(&GetData,&GetLength) == true);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.IsEmpty());
			assert(buffer.Dequeue(&GetData,&GetLength) == false);
			//test	void ClearAll(void);
			for(i=0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			buffer.ClearAll();
			presentNumber=0;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.IsEmpty());
			assert(buffer.Dequeue(&GetData,&GetLength) == false);

			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)//1
			{
				assert(buffer.Enqueue(&i,sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(buffer.Enqueue(&i,sizeof(i)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
		}
		//test pattern 2
		//Ministry copy vesion
		//test GetCount/IsEmpty/Peek/PeekLength/Enqueue/Dequeue(Ministry copy vesion)
		//
		int*   pWritePointer;
		{
			RingBuffer Buffer0c;
			assert(Buffer0c.Initialize(0,0,NULL) == false);
			assert(Buffer0c.Initialize(TEST_RING_BUFFER_CAPACITY, TEST_RING_BUFFER_ELEMENTSIZE,NULL) == true);
			assert(Buffer0c.IsEmpty() == true);
			assert(Buffer0c.GetCount() == 0);
			assert(Buffer0c.Dequeue(&GetLength) == false);
			assert(Buffer0c.PeekLength() == 0UL);
			additionalFrequency=0;
			maximumElements=0;
			overFlowFrequency=0;
			presentNumber=0;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			pWritePointer = (int *)Buffer0c.GetWriteBuffer();
			*pWritePointer = 0;
			assert(Buffer0c.Enqueue(sizeof(int)) == true);
			additionalFrequency++;
			maximumElements++;
			presentNumber++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			pWritePointer = (int *)Buffer0c.GetWriteBuffer();
			*pWritePointer = 1;
			assert(Buffer0c.Enqueue(sizeof(int)) == true);
			additionalFrequency++;
			maximumElements++;
			presentNumber++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer0c.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			///
			///
			///
			//RingBuffer Buffer=Buffer0c;
			RingBuffer Buffer;
			Buffer=Buffer0c;
			///
			///
			///
			Buffer.Dequeue(&GetLength);
			presentNumber--;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			Buffer.Dequeue(&GetLength);
			presentNumber--;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Enqueue test
			for(i=0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				pWritePointer = (int *)Buffer.GetWriteBuffer();
				*pWritePointer = i;
				assert(Buffer.Enqueue(sizeof(i)) == true);
				additionalFrequency++;
				presentNumber++;
			}
			maximumElements = TEST_RING_BUFFER_CAPACITY;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!Buffer.IsEmpty());
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			assert(Buffer.Enqueue(sizeof (int)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			// Dequeue test
			for(i = 0; i<TEST_RING_BUFFER_CAPACITY; i++)
			{
				PeekData = *(int *)Buffer.Peek(&GetLength);
				assert(PeekData == i);
				assert(Buffer.PeekLength() == GetLength);
				GetData = *(int *)Buffer.Dequeue(&GetLength);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(Buffer.IsEmpty());
			assert(Buffer.Dequeue(&GetLength) == false);
			// Enqueue and Dequeue test
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				pWritePointer = (int *)Buffer.GetWriteBuffer();
				*pWritePointer = i;
				assert(Buffer.Enqueue(sizeof (int)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!Buffer.IsEmpty());
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			assert(Buffer.Enqueue(sizeof (int)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				PeekData = *(int *)Buffer.Peek(&GetLength);
				assert(Buffer.PeekLength() == GetLength);
				GetData = *(int *)Buffer.Dequeue(&GetLength);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!Buffer.IsEmpty());
			for(i = 0; i < (TEST_RING_BUFFER_CAPACITY / 2); i++)
			{
				pWritePointer = (int *)Buffer.GetWriteBuffer();
				*pWritePointer = i;
				assert(Buffer.Enqueue(sizeof (int)) == true);
				additionalFrequency++;
				presentNumber++;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(!Buffer.IsEmpty());
			pWritePointer = (int *)Buffer.GetWriteBuffer();
			assert(Buffer.Enqueue(sizeof (int)) == false);
			overFlowFrequency++;
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			//Dequeue
			for(i = 0; i < TEST_RING_BUFFER_CAPACITY; i++)
			{
				PeekData = *(int *)Buffer.Peek(&GetLength);
				assert(Buffer.PeekLength() == GetLength);
				GetData = *(int *)Buffer.Dequeue(&GetLength);
				assert(PeekData == GetData);
				presentNumber--;
			}
#ifdef PAFEE_STATISTICS_ENABLE
			Buffer.GetStatistics(&statisticsRtn);
			assert(statisticsRtn.additionalFrequency == additionalFrequency);
			assert(statisticsRtn.maximumElements == maximumElements);
			assert(statisticsRtn.overFlowFrequency == overFlowFrequency);
			assert(statisticsRtn.presentNumber == presentNumber);
#endif
			assert(Buffer.IsEmpty());
			assert(Buffer.Dequeue(&GetLength) == false);
		}
	}
}
#endif //PAFEE_UNITTEST

}

