//****************************************************
//
//      project     PAFEE
//!      @file:     PafeeRingBufferT.h
//!      @author:   M.Mori
//!      @brief:    RingBufferT class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _PAFEE_RING_BUFFER_H_INCLUDED_
#define _PAFEE_RING_BUFFER_H_INCLUDED_
#include <new>
#include "PafeeObject.h"
#include "PafeeCommon.h"
#include "pafee_string.h"

namespace Pafee
{

//! RingBufferT
template<class T>
class RingBufferT : public Object
{
public:
	inline int AligmentSize(int size) {return ((((size)-1)|(((size)-1)^3))+1);};

#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct 
	{
		int additionalFrequency;
		int maximumElements;
		int overFlowFrequency;
		int presentNumber;
	} STATISTICS;
	void GetStatistics(
		STATISTICS* pStatistics ///< [out] Statistics information
	)
	{
		pafee_memcpy ( pStatistics, &m_Statistics, sizeof m_Statistics );
	}
private:
	STATISTICS m_Statistics;
#endif

private:
	typedef struct t_RingBufferPointer
	{
		T* p;
		t_RingBufferPointer* pNext;
		int Size;
	} RingBufferPointer;
	RingBufferPointer* m_pRingBufferPointer;
	RingBufferPointer* m_pWrite;
	RingBufferPointer* m_pRead;
public:
	RingBufferT(
		int capacity=0, ///< [in] and one element in can setting of the following as argument and ring buffer
		Allocator* pAllocator=NULL ///< [in] MemoryAllocator
	)
	{
		Initialize(capacity, pAllocator);
	}

	RingBufferT(
		RingBufferT& rbc, ///< [in] Initialize RingBuffer 
		Allocator* pAllocator=NULL ///< [in] MemoryAllocator
	)
	{
		Initialize(rbc.m_capacity, pAllocator);
		pafee_memcpy(this->m_pBuffer,rbc.m_pBuffer,rbc.m_bufferAllocateLength);
		{
//			T* pBuf = m_pBuffer;
			for(register int i=0;i<m_capacity;i++)
			{
				m_pRingBufferPointer[i].Size=rbc.m_pRingBufferPointer[i].Size;
			}
		}
		m_pRead  = rbc.m_pRead  - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
		m_pWrite = rbc.m_pWrite - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
		m_Statistics.additionalFrequency=rbc.m_Statistics.additionalFrequency;
		m_Statistics.maximumElements=rbc.m_Statistics.maximumElements;
		m_Statistics.overFlowFrequency=rbc.m_Statistics.overFlowFrequency;
		m_Statistics.presentNumber=rbc.m_Statistics.presentNumber;
#endif
	}

	~RingBufferT()
	{
		if ( m_pRingBufferPointer != NULL )
		{
			Deallocate(m_pRingBufferPointer);
		}
		if ( m_pBuffer != NULL )
		{
			T* p = m_pBuffer;
			for(int i=0;i<m_capacity;i++)
			{
				p->~T();
				p++;
			}
			Deallocate(m_pBuffer);
		}
	}

	bool Initialize(
		int capacity,  ///< [in] capacity
		Allocator* pAllocator=NULL ///< [in] MemoryAllocator
	)
	{
		SetAllocator(pAllocator);
		m_pRingBufferPointer = NULL;
		m_pBuffer = NULL;
		m_capacity = capacity;
		m_elementSize= sizeof(T);
		if ( (m_capacity == 0) || (m_elementSize == 0) )
		{
			return false;
		}
		m_bufferLength = m_capacity*m_elementSize;
		m_bufferAllocateLength = AligmentSize(m_bufferLength);
		m_pBuffer = (T*)Allocate(m_bufferAllocateLength);
		T* p = m_pBuffer;
		for(int i=0;i<m_capacity;i++)
		{
			new (p)T;
			p++;
		}
		m_pRingBufferPointer = (RingBufferPointer*)Allocate(capacity*sizeof(RingBufferPointer));
		{
			register T* pBuf = m_pBuffer;
			for(register int i=0;i<m_capacity;i++)
			{
				m_pRingBufferPointer[i].p=pBuf;
				m_pRingBufferPointer[i].pNext=&m_pRingBufferPointer[i+1];
				m_pRingBufferPointer[i].Size=0;
				pBuf++;
			}
			m_pRingBufferPointer[m_capacity-1].pNext=m_pRingBufferPointer;
		}
		m_pRead = m_pWrite = m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
		m_Statistics.additionalFrequency=0;
		m_Statistics.maximumElements=0;
		m_Statistics.overFlowFrequency=0;
		m_Statistics.presentNumber=0;
#endif
		return true;
	}

	RingBufferT& operator = (
		RingBufferT& rbc ///< [in] copied RingBufferT
	)
	{
		if ( m_pRingBufferPointer != NULL )
		{
			Deallocate(m_pRingBufferPointer);
		}
		if ( m_pBuffer != NULL )
		{
			T* p = m_pBuffer;
			for(int i=0;i<m_capacity;i++)
			{
				p->~T();
				p++;
			}
			Deallocate(m_pBuffer);
		}
		m_pRingBufferPointer = NULL;
		m_pBuffer = NULL;
		Initialize(rbc.m_capacity, m_pAllocator);
		pafee_memcpy(this->m_pBuffer,rbc.m_pBuffer,rbc.m_bufferAllocateLength);
		{
			//T* pBuf = m_pBuffer;
			for(register int i=0;i<m_capacity;i++)
			{
				m_pRingBufferPointer[i].Size=rbc.m_pRingBufferPointer[i].Size;
			}
		}
		m_pRead  = rbc.m_pRead  - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
		m_pWrite = rbc.m_pWrite - rbc.m_pRingBufferPointer + m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
		m_Statistics.additionalFrequency=rbc.m_Statistics.additionalFrequency;
		m_Statistics.maximumElements=rbc.m_Statistics.maximumElements;
		m_Statistics.overFlowFrequency=rbc.m_Statistics.overFlowFrequency;
		m_Statistics.presentNumber=rbc.m_Statistics.presentNumber;
#endif
		return *this;
	}

	int GetCount()
	{
		register int usedCount=0;
		register RingBufferPointer* p=m_pRingBufferPointer;
		for(register int i=0;i<m_capacity;i++)
		{
			usedCount+=p->Size;
			p++;
		}
		return usedCount;
	}

	bool IsEmpty(void)
	{
		register int usedCount=0;
		register RingBufferPointer* p=m_pRingBufferPointer;
		for(register int i=0;i<m_capacity;i++)
		{
			usedCount+=p->Size;
			p++;
		}
		return usedCount==0;
	}

	bool Enqueue (
		const T& Data  ///< [in] Data
	)
	{
		if ( m_pWrite->Size == 1 )
		{
#ifdef PAFEE_STATISTICS_ENABLE
			m_Statistics.overFlowFrequency++;
#endif
			return false;
		}
		*m_pWrite->p = Data;
		m_pWrite->Size = 1;
		m_pWrite=m_pWrite->pNext;
#ifdef PAFEE_STATISTICS_ENABLE
		m_Statistics.additionalFrequency++;
		m_Statistics.maximumElements++;
		if(m_capacity < m_Statistics.maximumElements)
		{
			m_Statistics.maximumElements = m_capacity;
		}
		m_Statistics.presentNumber++;
#endif
		return true;
	}
	bool Dequeue(
		T& Buffer ///< [out] Prepared Buffer
	)
	{
		if ( m_pRead->Size == 0 )
		{
			return false;
		}
		Buffer = *m_pRead->p;
		m_pRead->Size = 0;
		m_pRead=m_pRead->pNext;
#ifdef PAFEE_STATISTICS_ENABLE
		m_Statistics.presentNumber--;
#endif
		return true;
	}

	bool Peek(
		T& Buffer ///< [out] Prepared Buffer
	)
	{
		if ( m_pRead->Size == 0 )
		{
			return false;
		}
		Buffer = *m_pRead->p;
		return true;
	}

	void ClearAll(void)
	{
		for(int i=0;i<m_capacity;i++)
		{
			m_pRingBufferPointer[i].Size=0;
		}
		m_pRead = m_pWrite = m_pRingBufferPointer;
#ifdef PAFEE_STATISTICS_ENABLE
		m_Statistics.presentNumber=0;
#endif
		return;
	}

private:
	T* m_pBuffer;
	int m_capacity;
	int m_elementSize;
	int m_bufferLength;
	int m_bufferAllocateLength;

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest(void);
#endif // PAFEE_UNITTEST

};

}
#endif //_PAFEE_RING_BUFFER_H_INCLUDED_

