//****************************************************
//
//    project   Pafee
//!   @file:    PafeeStack.h
//!   @author:  M.Tanaka
//!   @brief:   Stack
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "PafeeStack.h"

namespace Pafee
{

//----------------------------------------
//! Default Constructor
//----------------------------------------
Stack::Stack(
	Allocator* pAllocator ///< [in] Memory Allocator
)
{
	SetAllocator(pAllocator);
	m_pList = (List*)Allocate(sizeof(List));
	new (m_pList)List(pAllocator);
}

//----------------------------------------
//! Copy Constructor
//----------------------------------------
Stack::Stack(
	const Stack& stkc, ///< [in] initial value of Stack
	Allocator* pAllocator ///< [in] Memory Allocator
)
{
	SetAllocator(pAllocator);
	m_pList = (List*)Allocate(sizeof(List));
	new (m_pList)List(pAllocator);
	*m_pList = *stkc.m_pList;
}

//----------------------------------------
//! Destructor
//----------------------------------------
Stack::~Stack()
{
	m_pList->~List();
	Deallocate(m_pList); // delete m_pList;
}

//-------------------------------------------------------------------------------------
//! operator =
//! @return Result(the address of Stack)
//-------------------------------------------------------------------------------------
Stack& Stack::operator =(
	const Stack& stkc ///< [in] Copied Stack
)
{
	*m_pList = *stkc.m_pList;
	return *this;
}

//----------------------------------------
//! Pop
//! @return: Result(TRUE:success FALSE: failure)
//----------------------------------------
bool Stack::Pop(
	void* pBuffer, ///< [out] Head Position Element 
	int* pLength, ///< [out] Head Position Element Size
	int bufSize ///< [in] Prepared buffer sizes 
)
{
	if(m_pList->GetFront(pBuffer,pLength,bufSize))
	{
		m_pList->RemoveFront();
		return true;
	}
	return false;
}

#ifdef PAFEE_STATISTICS_ENABLE
//----------------------------------------
//! GetStatistics
//! @return: void
//----------------------------------------
void Stack::GetStatistics(
	Stack::STATISTICS* pStatistics ///< [out] statistics Pointer
)
{
	Pafee::List::STATISTICS statisticsRtn;
	m_pList->GetStatistics(&statisticsRtn);
	m_Statistics.addCount = statisticsRtn.addCount;
	m_Statistics.maxElementCount = statisticsRtn.maxElementCount;
	m_Statistics.maxElementSize = statisticsRtn.maxElementSize;
	m_Statistics.minElementSize = statisticsRtn.minElementSize;
	m_Statistics.presentElementCount = statisticsRtn.presentElementCount;
	pafee_memcpy ( pStatistics, &m_Statistics, sizeof (Stack::STATISTICS) );
}
#endif //PAFEE_STATISTICS_ENABLE

#ifdef PAFEE_UNITTEST
//----------------------------------------------------------------------------
//!	Unit test
//! @return: void
//----------------------------------------------------------------------------
typedef struct tagData
{
	char data[10];	//addback,addfront
}
TDATA;

typedef struct tagData2
{
	char data2[15]; //setAt
}
TDATA2;

typedef struct tagData3
{
	char data3[20];	//getWrite
}
TDATA3;

void Stack::UnitTest(void)
{
	{
		//STATISTICS
		int addCount=0;
		int maxElementCount=0;
		int minElementSize=0;
		int maxElementSize=0;
		int presentElementCount=0;
#ifdef PAFEE_STATISTICS_ENABLE
		Stack::STATISTICS statisticsRtn;
#endif
	//#define TEST_STACK_CAPACITY	10
		const int TEST_STACK_CAPACITY = 10;
//		void* padr=NULL;
		int nagasa;
		int i;
		char* p;
		TDATA tempData;
		pafee_memset(tempData.data,'1',sizeof(TDATA));
		TDATA2 tempData2;
		pafee_memset(tempData2.data2,'\0',sizeof(TDATA2));
		TDATA3 tempData3;
		pafee_memset(tempData3.data3,'\0',sizeof(TDATA3));
	///	Stack stc(10);
		Stack stc;
		int bufferSize1 = sizeof(TDATA);
//		int bufferSize2 = sizeof(TDATA2);
		int bufferSize3 = sizeof(TDATA3);
	//	assert(stc.Initialize(TEST_STACK_CAPACITY) == TRUE);
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test Push/GetCount/IsEmpty
		assert(stc.GetCount() == 0);
		assert(stc.IsEmpty());
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Push((void* )&tempData,sizeof(TDATA)));
			addCount++;
			presentElementCount++;
		}
		maxElementCount = TEST_STACK_CAPACITY;
		minElementSize = bufferSize1;
		maxElementSize = bufferSize1;
		assert(stc.GetCount() == TEST_STACK_CAPACITY);
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//no copy version(GetWriteBuffer,Push,Peek,Pop)
		p = (char* )stc.GetWriteBuffer(sizeof(TDATA));
		assert(p!=NULL);
		pafee_memcpy(p,"TESTXYZ",7);
		stc.Push(7);
		addCount++;
		presentElementCount++;
		maxElementCount++;
		minElementSize = 7;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		pafee_memcpy(tempData3.data3,(char* )stc.Peek(&nagasa),7);
		tempData3.data3[7] = '\0';
		assert(pafee_strcmp(tempData3.data3,"TESTXYZ")==0);
		assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		presentElementCount--;
		minElementSize = bufferSize1;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test Peek
		assert(stc.Peek((void* )&tempData3,&nagasa,bufferSize3));
		assert(nagasa != 0);
		assert(stc.Peek(&nagasa));	//no copy version
		assert(nagasa != 0);
		//test PeekLength
		assert(stc.PeekLength() == sizeof(TDATA));
		//test Pop
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
			presentElementCount--;
		}
		minElementSize = 0;
		maxElementSize = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		assert(stc.GetCount() == 0);
		assert(stc.IsEmpty());
		//test push
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Push((void* )&tempData,sizeof(TDATA)));
			addCount++;
			presentElementCount++;
		}
		minElementSize = bufferSize1;
		maxElementSize = bufferSize1;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(stc.Push((void* )&tempData,sizeof(TDATA)));
		addCount++;
		presentElementCount++;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(stc.Pop((void* )&tempData3,&nagasa, bufferSize3));
		presentElementCount--;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test RemoveFront
		stc.RemoveFront();
		presentElementCount--;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		for(i=0;i<TEST_STACK_CAPACITY-1;i++)
		{
			assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
			presentElementCount--;
		}
		minElementSize = 0;
		maxElementSize = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Push((void* )&tempData,sizeof(TDATA)));
			addCount++;
			presentElementCount++;
		}
		minElementSize = bufferSize1;
		maxElementSize = bufferSize1;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test RemoveBack
		stc.RemoveBack();
		presentElementCount--;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		for(i=0;i<TEST_STACK_CAPACITY-1;i++)
		{
			assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
			presentElementCount--;
		}
		minElementSize = 0;
		maxElementSize = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		//test RemoveAll
		stc.RemoveAll();
		minElementSize = 0;
		maxElementSize = 0;
		presentElementCount = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		assert(stc.GetCount() == 0);
		assert(stc.IsEmpty());
	}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
	{
		//STATISTICS
		int addCount=0;
		int maxElementCount=0;
		int minElementSize=0;
		int maxElementSize=0;
		int presentElementCount=0;
#ifdef PAFEE_STATISTICS_ENABLE
		Stack::STATISTICS statisticsRtn;
#endif
		const int TEST_STACK_CAPACITY = 10;
//		void* padr=NULL;
		int nagasa;
		int i;
		char* p;
		TDATA tempData;
		pafee_memset(tempData.data,'1',sizeof(TDATA));
		TDATA2 tempData2;
		pafee_memset(tempData2.data2,'\0',sizeof(TDATA2));
		TDATA3 tempData3;
		pafee_memset(tempData3.data3,'\0',sizeof(TDATA3));
		Stack stc0c;
		int bufferSize1 = sizeof(TDATA);
//		int bufferSize2 = sizeof(TDATA2);
		int bufferSize3 = sizeof(TDATA3);
#ifdef PAFEE_STATISTICS_ENABLE
		stc0c.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test Push/GetCount/IsEmpty
		assert(stc0c.GetCount() == 0);
		assert(stc0c.IsEmpty());
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc0c.Push((void* )&tempData,sizeof(TDATA)));
			addCount++;
			presentElementCount++;
		}
		maxElementCount = TEST_STACK_CAPACITY;
		minElementSize = bufferSize1;
		maxElementSize = bufferSize1;
		assert(stc0c.GetCount() == TEST_STACK_CAPACITY);
#ifdef PAFEE_STATISTICS_ENABLE
		stc0c.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//no copy version(GetWriteBuffer,Push,Peek,Pop)
		p = (char* )stc0c.GetWriteBuffer(sizeof(TDATA));
		assert(p!=NULL);
		pafee_memcpy(p,"TESTXYZ",7);
		stc0c.Push(7);
		addCount++;
		presentElementCount++;
		maxElementCount++;
		minElementSize = 7;
#ifdef PAFEE_STATISTICS_ENABLE
		stc0c.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		pafee_memcpy(tempData3.data3,(char* )stc0c.Peek(&nagasa),7);
		tempData3.data3[7] = '\0';
		assert(pafee_strcmp(tempData3.data3,"TESTXYZ")==0);
		assert(stc0c.Pop((void* )&tempData3,&nagasa,bufferSize3));
		presentElementCount--;
		minElementSize = bufferSize1;
#ifdef PAFEE_STATISTICS_ENABLE
		stc0c.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//Stack stc=stc0c;
		Stack stc;
		stc=stc0c;
		//test Peek
		assert(stc.Peek((void* )&tempData3,&nagasa,bufferSize3));
		assert(nagasa != 0);
		assert(stc.Peek(&nagasa));	//no copy version
		assert(nagasa != 0);
		//test PeekLength
		assert(stc.PeekLength() == sizeof(TDATA));
		//test Pop
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
			presentElementCount--;
		}
		minElementSize = 0;
		maxElementSize = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		assert(stc.GetCount() == 0);
		assert(stc.IsEmpty());

		//test push
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Push((void* )&tempData,sizeof(TDATA)));
			addCount++;
			presentElementCount++;
		}
		minElementSize = bufferSize1;
		maxElementSize = bufferSize1;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(stc.Push((void* )&tempData,sizeof(TDATA)));
		addCount++;
		presentElementCount++;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(stc.Pop((void* )&tempData3,&nagasa, bufferSize3));
		presentElementCount--;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test RemoveFront
		stc.RemoveFront();
		presentElementCount--;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		for(i=0;i<TEST_STACK_CAPACITY-1;i++)
		{
			assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
			presentElementCount--;
		}
		minElementSize = 0;
		maxElementSize = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		for(i=0;i<TEST_STACK_CAPACITY;i++)
		{
			assert(stc.Push((void* )&tempData,sizeof(TDATA)));
			addCount++;
			presentElementCount++;
		}
		minElementSize = bufferSize1;
		maxElementSize = bufferSize1;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		//test RemoveBack
		stc.RemoveBack();
		presentElementCount--;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		for(i=0;i<TEST_STACK_CAPACITY-1;i++)
		{
			assert(stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
			presentElementCount--;
		}
		minElementSize = 0;
		maxElementSize = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		//test RemoveAll
		stc.RemoveAll();
		minElementSize = 0;
		maxElementSize = 0;
		presentElementCount = 0;
#ifdef PAFEE_STATISTICS_ENABLE
		stc.GetStatistics(&statisticsRtn);
		assert(statisticsRtn.addCount == addCount);
		assert(statisticsRtn.maxElementCount == maxElementCount);
		assert(statisticsRtn.minElementSize == minElementSize);
		assert(statisticsRtn.maxElementSize == maxElementSize);
		assert(statisticsRtn.presentElementCount == presentElementCount);
#endif
		assert(!stc.Pop((void* )&tempData3,&nagasa,bufferSize3));
		assert(stc.GetCount() == 0);
		assert(stc.IsEmpty());
	}

} //UnitTest
#endif

} //namespace
