//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	PafeeRingBufferedListener.cpp
//! 	 @author:	Takatsura Miyawaki
//! 	 @brief:	Event Listener with Ring buffer
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//


#include "PafeeRingBufferedListener.h"

using namespace Pafee;


//--------------------------------------------------------------
//! Constructor
//--------------------------------------------------------------
RingBufferedListener::RingBufferedListener()
{
	m_isKeepRunning = true;
	m_isRunning = false;
};

//--------------------------------------------------------------
//! Destructor
//--------------------------------------------------------------
RingBufferedListener::~RingBufferedListener()
{
	m_isKeepRunning = false;
#ifndef PAFEE_OS_ITRON
	PafeeTask::Delete();
	m_signal.Delete();
	m_mutex.Delete();
#endif
};

//--------------------------------------------------------------
//! Intialize
//--------------------------------------------------------------
#ifdef PAFEE_OS_ITRON
bool RingBufferedListener::Initialize(
							int capacity,			///< [in] capacity of ring buffer (max element count)
							int elementSize,		///< [in] element size of ring buffer 
							ID taskID,				///< [in] task id
							ID signalID,			///< [in] signal id
							ID mutexID,				///< [in] mutex id
							Allocator* pAllocator	///< [in] allocator for ring buffer
							)
{
	if(m_signal.Attach(signalID) == false)
	{
		return false;
	}
	if(m_mutex.Attach(mutexID) == false)
	{
		return false;
	}
	if(PafeeTask::Attach(taskID) == false)
	{
		return false;
	}
	m_ringBuffer.Initialize(capacity, elementSize, pAllocator);
	return PafeeTask::Start();
}
#endif
#if defined(PAFEE_OS_WINDOWS) || defined(PAFEE_OS_LINUX)

bool RingBufferedListener::Initialize(
							int capacity,				///< [in] capacity of ring buffer (max element count)
							int elementSize,			///< [in] element size of ring buffer 
							TASK_PRIORITY priority,		///< [in] task priority
							TASK_SCHEDULING schedule,	///< [in] task scheduling (linux only, set SCHED_XXXX, otherwise set 0)
							int stackSize,				///< [in] stack size for task
							Allocator* pAllocator		///< [in] allocator for ring buffer
							)
{
	if(m_signal.Create() == false)
	{
		return false;
	}
	if(m_mutex.Create() == false)
	{
		return false;
	}
	
	if(PafeeTask::Create(&m_taskID, priority, schedule, stackSize, NULL, true) == false)
	{
		return false;
	}
	m_ringBuffer.Initialize(capacity, elementSize, pAllocator);
#if defined(PAFEE_OS_WINDOWS)
	return PafeeTask::Start();
#else
	return true;
#endif
}

#endif

//--------------------------------------------------------------
//! Terminate 
//--------------------------------------------------------------
void RingBufferedListener::Terminate(void)
{
	m_isKeepRunning = false;
	while(m_isRunning)
	{
		tslp_tsk(100);
	}
}

//--------------------------------------------------------------
//! Send event
//! @return result (true:success false:failure)
//--------------------------------------------------------------
bool RingBufferedListener::SendEvent(
						unsigned long eventID,///< [in] event id
						const void* pParam,	///< [in] parameter buffer
						int paramLength		///< [in] parameter length
						)
{
	int length = sizeof(eventID) + paramLength;

	// Enqueue event and parameter
	m_mutex.Lock();
	unsigned char* pBuffer = (unsigned char*) m_ringBuffer.GetWriteBuffer();
	if(pBuffer == NULL)
	{
		m_mutex.Unlock();
		return false;
	}
	*((unsigned long*) pBuffer) = eventID;
	pafee_memcpy(pBuffer+sizeof(eventID), pParam, paramLength);
	m_ringBuffer.Enqueue(length);
	m_mutex.Unlock();

	// Kick the event handler
	m_signal.Send();
	return true;
}

//--------------------------------------------------------------
//! Task procedure
//--------------------------------------------------------------
void* RingBufferedListener::TaskProcedure(
						void* pParam	///< [in] task parameter
						)
{
	unsigned char* pBuffer;
	int length;
	
	m_isRunning = true;
	while(m_isKeepRunning)
	{
		m_signal.Wait();
		if(m_isKeepRunning)
		{
			m_mutex.Lock();
			while(m_isKeepRunning)
			{
				pBuffer = (unsigned char*) m_ringBuffer.Peek(&length);
				if(pBuffer && m_isKeepRunning)
				{
					// Handle event
					unsigned long eventID = *((unsigned long*) pBuffer);
					OnEventReceived(eventID, pBuffer+sizeof(eventID), length-sizeof(eventID));
					m_ringBuffer.Dequeue(&length);
				}
				else
				{
					break;
				}
			}
			m_mutex.Unlock();
		}
	}
	m_isRunning = false;
	return NULL;
}

