//********************************************************************
//
//! 	 @file: 	PafeeComboAllocator.cpp
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Combo allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "PafeeCommon.h"
#include "PafeeComboAllocator.h"

using namespace Pafee;


//------------------------------------------------
//! Constractor
//------------------------------------------------
ComboAllocator::ComboAllocator()
{
	for(int i = 0; i < MAX_ALLOCATOR; i++)
	{
		m_allocs[i].minAllocationSize = 0;
		m_allocs[i].maxAllocationSize = 0;
		m_allocs[i].pAllocator = NULL;
	}
	m_pHeadAlloc = &m_allocs[0];
	m_pTailAlloc = NULL;
}

//------------------------------------------------
//! Destoractor
//------------------------------------------------
ComboAllocator::~ComboAllocator()
{
}


//------------------------------------------------
//! Allocate memory
// @return allocated pointer
//------------------------------------------------
inline void* ComboAllocator::Allocate(
									unsigned long size	///< [in] size to allocate
									)
{
	ALLOC *pAlloc = m_pHeadAlloc;
	while(pAlloc)
	{
		if(pAlloc->pAllocator)
		{
			if(	(pAlloc->minAllocationSize <= size) &&
				size <= (pAlloc->maxAllocationSize))
			{
				void* ptr = pAlloc->pAllocator->Allocate(size);
				if(ptr)
				{
					return ptr;
				}
				// try next allocator
			}
		}
		pAlloc++;
	}

/*
	for(int i = 0; i < MAX_ALLOCATOR; i++)
	{
		if(m_allocs[i].pAllocator)
		{
			if(	(m_allocs[i].minAllocationSize <= size) &&
				size <= (m_allocs[i].maxAllocationSize))
			{
				void* ptr = m_allocs[i].pAllocator->Allocate(size);
				if(ptr)
				{
					return ptr;
				}
				else
				{
					// try next allocator
				}
			}
		}
		else
		{
			break;
		}
	}
*/
	return NULL;
}

//------------------------------------------------
//! Deallocate memory
// @return void
//------------------------------------------------
inline void ComboAllocator::Deallocate(
									void* ptr	///< [in] poiner to deallocate
									)
{
	ALLOC *pAlloc = m_pHeadAlloc;
	while(pAlloc)
	{
		if(pAlloc->pAllocator)
		{
			if(pAlloc->pAllocator->IsOwnAddress(ptr))
			{
				pAlloc->pAllocator->Deallocate(ptr);
				break;
			}
		}
		pAlloc++;
	}
/*
	for(int i = 0; i < MAX_ALLOCATOR; i++)
	{
		if(m_allocs[i].pAllocator)
		{
			if(m_allocs[i].pAllocator->IsOwnAddress(ptr))
			{
				m_allocs[i].pAllocator->Deallocate(ptr);
				break;
			}
		}
		else
		{
			break;
		}
	}
*/
}

//------------------------------------------------
//! Check given address is in own address range
// @return true: own address, false:not own address
//------------------------------------------------
bool ComboAllocator::IsOwnAddress(
								void* ptr	///< [in] pointer to check
								)
{
	for(int i = 0; i < MAX_ALLOCATOR; i++)
	{
		if(m_allocs[i].pAllocator)
		{
			if(m_allocs[i].pAllocator->IsOwnAddress(ptr) == true)
			{
				return true;
			}
		}
		else
		{
			break;
		}
	}
	return false;
}

//------------------------------------------------
//! Add allocator
// @return true: success false: failed
//------------------------------------------------
bool ComboAllocator::AddAllocator(
								Allocator* pAllocator,				///< [in] Allocator
								unsigned long minAllocationSize,	///< [in] minimum allocation size of the allocator
								unsigned long maxAllocationSize		///< [in] maximum allocation size of the allocator
								)
{
	for(int i = 0; i < MAX_ALLOCATOR; i++)
	{
		if(m_allocs[i].pAllocator == NULL)
		{
			m_allocs[i].minAllocationSize = minAllocationSize;
			m_allocs[i].maxAllocationSize = maxAllocationSize;
			m_allocs[i].pAllocator = pAllocator;
			m_pTailAlloc = &m_allocs[i];
			return true;
		}
	}
	return false;
}

#ifdef PAFEE_STATISTICS_ENABLE

//------------------------------------------------
//! Get statistics
//------------------------------------------------
void ComboAllocator::GetStatistics(
								StreamOutput& stream				///< [in/out] stream output
								)
{
	for(int i = 0; i < MAX_ALLOCATOR; i++)
	{
		if(m_allocs[i].pAllocator)
		{
			m_allocs[i].pAllocator->GetStatistics(stream);
		}
	}
}

#endif

