//********************************************************************
//
//! 	 @file: 	PafeeLockedAllocator.cpp
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Locked memory allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifdef PAFEE_DEBUG
#include <assert.h>
#endif
#ifdef PAFEE_EXCEPTION
#include <new>
#include <stdexcept>
#endif

#include "PafeeLockedAllocator.h"

using namespace Pafee;

#ifdef PAFEE_DEBUG
#define PAFEE_ASSERT(CONDITION)	assert((CONDITION));
#else
#define PAFEE_ASSERT(CONDITION)
#endif

//-------------------------------------------------
//! Constractor
//-------------------------------------------------
LockedAllocator::LockedAllocator(Allocator* pAllocator /*=NULL*/)
{
	m_pAllocator = pAllocator;
	m_mutex.Create();
}

//-------------------------------------------------
//! Destoractor
//-------------------------------------------------
LockedAllocator::~LockedAllocator()
{
	m_mutex.Delete();
}

//-------------------------------------------------
//! Initialize
//-------------------------------------------------
void LockedAllocator::Initialize(
								Allocator* pAllocator	///< [in] Target Allocator for mutual exclusion
								)
{
	m_pAllocator = pAllocator;
}
//-------------------------------------------------
//! Allocate memory
//-------------------------------------------------
void* LockedAllocator::Allocate(
								unsigned long size	///< [in] size to allocate
								)
{
	void* ptr = NULL;
	PAFEE_ASSERT(m_pAllocator);
#ifdef PAFEE_EXCEPTION
	m_mutex.Lock();
	try
	{
		ptr = m_pAllocator->Allocate(size);
	}
	catch(std::exception& e)
	{
		m_mutex.Unlock();
		throw e;
	}
	catch(...)
	{
		m_mutex.Unlock();
		throw std::bad_alloc();
	}
		m_mutex.Unlock();
#else
	m_mutex.Lock();
	ptr = m_pAllocator->Allocate(size);
	m_mutex.Unlock();
#endif
	return ptr;
}

//-------------------------------------------------
//! Deallocate memory
//-------------------------------------------------
void LockedAllocator::Deallocate(
								void* ptr	///< [in] poiner to deallocate
								)
{
	PAFEE_ASSERT(m_pAllocator);
#ifdef PAFEE_EXCEPTION
	m_mutex.Lock();
	try
	{
		m_pAllocator->Deallocate(ptr);
	}
	catch(std::exception& e)
	{
		m_mutex.Unlock();
		throw e;
	}
	catch(...)
	{
		m_mutex.Unlock();
		throw std::exception();
	}
	m_mutex.Unlock();
#else
	m_mutex.Lock();
	m_pAllocator->Deallocate(ptr);
	m_mutex.Unlock();
#endif
}

//-------------------------------------------------
//! Check given address is in own address range
//-------------------------------------------------
inline bool LockedAllocator::IsOwnAddress(
										void* ptr	///< [in] pointer to check
										)
{
	return m_pAllocator->IsOwnAddress(ptr);
}

#ifdef PAFEE_STATISTICS_ENABLE
//-------------------------------------------------
//! Get statistics
//-------------------------------------------------
void LockedAllocator::GetStatistics(StreamOutput& stream)
{
	m_pAllocator->GetStatistics(stream);
}
#endif

