//********************************************************************
//
//! 	 @file: 	PafeeStatisticsAllocator.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Statistics memory allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _STATISTICS_ALLOCATOR_H_INCLUDED_
#define _STATISTICS_ALLOCATOR_H_INCLUDED_

#include "PafeeCommon.h"
#include "pafee_string.h"
#include "PafeeAllocator.h"
#include "PafeeStreamOutput.h"

namespace Pafee {

//! Statistics Allocator
//
//! Records allocation size and frequency up to MAX_RECORD_COUNT,
//! target allocation size range is customizable by SetResolution()
class StatisticsAllocator : public Allocator
{
public:
	enum {MAX_RECORD_COUNT = 256};
	typedef struct _STATISTICS
	{
		struct
		{
			unsigned int allocateSize;
			unsigned int allocateCount;
		} usage[MAX_RECORD_COUNT];

		unsigned int totalAllocateRequests;
		unsigned int totalDeallocateRequests;
		unsigned int overflow;
		
	} STATISTICS;

public:
	//! Constractor
	StatisticsAllocator(Allocator* pBackendAllocator=NULL);

	//! Destoractor
	virtual ~StatisticsAllocator();
	
	//! Initialize
	void Initialize(int resolutionBits, Allocator* pAllocator = NULL);
	
	//! Allocate memory
	virtual void* Allocate(unsigned long size);

	//! Deallocate memory
	virtual void Deallocate(void* ptr);

	//! Check given address is in own address range
	virtual bool IsOwnAddress(void* ptr);

#ifdef PAFEE_STATISTICS_ENABLE
	//! Get statistics
	virtual void GetStatistics(StreamOutput& stream);
#endif

	//! Get statistics
	void GetStatistics(StatisticsAllocator::STATISTICS* pStat);


	//! SetAllocator
	void SetAllocator(Allocator* pAllocator);

	//! Set record resolution
	void SetResolution(int resolutionBits);


protected:
	//! Update statistics
	void UpdateStatistics(size_t allocateSize);

	//! Swap data
	void Swap(unsigned int& dataA, unsigned int& dataB);

protected:
	
	static const unsigned int EMPTY;	///< empty mark
	unsigned int m_resolution;			///< record resolution bits
	STATISTICS m_stat;					///< statistics
	Allocator*	m_pBackendAllocator;	///< backend allocator
	
private:
	//! Copy constractor
	StatisticsAllocator(const StatisticsAllocator& src);

	//! Operator equal
	StatisticsAllocator& operator=(const StatisticsAllocator& src);
};

};

#endif // _STATISTICS_ALLOCATOR_H_INCLUDED_
