//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	Mutex.c
//! 	 @author:	Technocrats
//! 	 @brief:	Mutex class
//
//********************************************************************
//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND noneINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include <limits.h>
#include "PafeeMutex.h"

#ifdef PAFEE_OS_LINUX
#include <sys/time.h>
#include <sys/errno.h>
#endif // PAFEE_OS_LINUX

#ifdef __cplusplus
namespace Pafee{
#endif

#ifdef PAFEE_OS_WINDOWS
/**
 * @brief  Mutex class constructor for Windows
 * @param  none
 * @return none
 */
PafeeMutex::PafeeMutex(
)
{
	m_handle = INVALID_HANDLE_VALUE;
	m_error = ERROR_NONE;

	MUTEX_DEBUG(("Constractor\n"));
}

/**
 * @brief  Mutex class destructor
 * @param  none
 * @return none
 */
PafeeMutex::~PafeeMutex(
)
{
	Delete();

	MUTEX_DEBUG(("Destractor\n"));
}

/**
 * @brief  Make new mutex for Windows
 * @param  none
 * @return Create success or not
 * @retval true:create success
 * @retval false:create failed
 */
bool PafeeMutex::Create(
)
{
	if(m_handle != INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("Create faild : m_handle = %d\n", m_handle));

		m_error = ERROR_HAS_MUTEX;
		return false;
	}

	m_handle = CreateMutex(NULL, false, NULL);

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("Create faild : m_handle = %d\n", m_handle));

		m_error = ERROR_CANNOT_CREATE;
		return false;
	}
	else
	{
		MUTEX_DEBUG(("Create succeeded : m_handle = %d\n", m_handle));

		return true;
	}
}

/**
 * @brief  Delete mutex for Windows
 * @param  none
 * @return Delete success or not
 * @retval true:Delete success
 * @retval false:Delete failed
 */
bool PafeeMutex::Delete(
)
{
	int returnValue;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("Delete failed : m_handle = %d\n", m_handle));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	returnValue = CloseHandle(m_handle);

	if(returnValue == 0)
	{
		MUTEX_DEBUG(("Delete failed : m_handle = %d\n", m_handle));
		m_error = ERROR_CANNOT_DELETE;
		return false;
	}
	else
	{
		MUTEX_DEBUG(("Delete success : m_handle = %d\n", m_handle));
		m_handle = INVALID_HANDLE_VALUE;
		return true;
	}
}

/**
 * @brief  Lock. If can not lock, wait until get lock for Windows
 * @param  none
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::Lock(
)
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("Lock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	errorCode = WAIT_TIMEOUT;
	while(errorCode == WAIT_TIMEOUT)
	{
		errorCode = WaitForSingleObject(m_handle, ULONG_MAX);
		switch(errorCode)
		{
		case WAIT_OBJECT_0:
		case WAIT_ABANDONED:
			MUTEX_DEBUG(("Lock success : m_handle = %d\n", m_handle));
			return true;
			break;

		case WAIT_TIMEOUT:
		case WAIT_FAILED:
		default:
			break;
		}
	}

	return false;
}

/**
 * @brief  Async lock. If can not lock, return false for Windows
 * @param  none
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::AsyncLock(
)
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("AsyncLock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	errorCode = WaitForSingleObject(m_handle, 0);

	switch(errorCode)
	{
	case WAIT_OBJECT_0:
	case WAIT_ABANDONED:
		MUTEX_DEBUG(("AsyncLock success : m_handle = %d\n", m_handle));
		return true;
		break;

	case WAIT_TIMEOUT:
//		m_error = ERROR_TIME_OUT;
//		return false;
//		break;
	case WAIT_FAILED:
	default:
		MUTEX_DEBUG(("AsyncLock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Timed lock. If can not lock, wait until times up for Windows
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::TimedLock(
	unsigned long time ///< [in] Timeout time
)
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("TimedLock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	errorCode = WaitForSingleObject(m_handle, time);

	switch(errorCode)
	{
	case WAIT_OBJECT_0:
	case WAIT_ABANDONED:
		MUTEX_DEBUG(("TimedLock success : m_handle = %d\n", m_handle));
		return true;
		break;

	case WAIT_TIMEOUT:
		MUTEX_DEBUG(("TimedLock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_TIME_OUT;
		return false;
		break;
	case WAIT_FAILED:
	default:
		MUTEX_DEBUG(("TimedLock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Unlock. If can not unlock, return false for Windows
 * @param  none
 * @return Unlock success or not
 * @retval true:Unlock success
 * @retval false:Unlock failed
 */
bool PafeeMutex::Unlock(
)
{
	int returnValue;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		MUTEX_DEBUG(("Unlock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	returnValue = ReleaseMutex(m_handle);

	if(returnValue == 0)
	{
		MUTEX_DEBUG(("Unlock failed : m_handle = %d\n", m_handle));
		m_error = ERROR_CANNOT_UNLOCK;
		return false;
	}
	else
	{
		MUTEX_DEBUG(("Unlock success : m_handle = %d\n", m_handle));
		return true;
	}
}

/**
 * @brief  Get error code. 
 * @param  none
 * @return Mutex error code
 * @retval MutexError
 */
PafeeMutex::MUTEX_ERROR PafeeMutex::GetLastError(
)
{
	return m_error;
}

#endif // PAFEE_OS_WINDOWS

#ifdef PAFEE_OS_LINUX
/**
 * @brief  Mutex class constructor for Linux
 * @param  none
 * @return none
 */
PafeeMutex::PafeeMutex(
)
{
	m_created = false;
	m_error = ERROR_NONE;

	MUTEX_DEBUG(("Constractor\n"));
}

/**
 * @brief  Mutex class destructor
 * @param  none
 * @return none
 */
PafeeMutex::~PafeeMutex(
)
{
	Delete();

	MUTEX_DEBUG(("Destractor\n"));
}

/**
 * @brief  Make new mutex for Linux
 * @param  none
 * @return Create success or not
 * @retval true:create success
 * @retval false:create failed
 */
bool PafeeMutex::Create(
)
{
	if(m_created == true)
	{
		MUTEX_DEBUG(("Create failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_HAS_MUTEX;
		return false;
	}

	pthread_mutex_init(&m_mutex, NULL); // default : fast

	MUTEX_DEBUG(("Create success : m_mutex = %d\n", &m_mutex));
	m_created = true;

	return true;
}

/**
 * @brief  Delete mutex for Linux
 * @param  none
 * @return Delete success or not
 * @retval true:Delete success
 * @retval false:Delete failed
 */
bool PafeeMutex::Delete(
)
{
	int returnValue;

	if(m_created == false)
	{
		MUTEX_DEBUG(("Delete failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	returnValue = pthread_mutex_destroy(&m_mutex);

	switch(returnValue)
	{
	case 0:
		MUTEX_DEBUG(("Delete success : m_mutex = %d\n", &m_mutex));
		m_created = false;
		return true;
		break;
	case EBUSY:
	default:
		MUTEX_DEBUG(("Delete failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_CANNOT_DELETE;
		return false;
		break;
	}
}

/**
 * @brief  Lock. If can not lock, wait until get lock for Linux
 * @param  none
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::Lock(
)
{
	int errorCode;

	if(m_created == false)
	{
		MUTEX_DEBUG(("Lock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	errorCode = pthread_mutex_lock(&m_mutex);
	switch(errorCode)
	{
	case 0:
		MUTEX_DEBUG(("Lock success : m_mutex = %d\n", &m_mutex));
		return true;
		break;
	case EINVAL:
	case EDEADLK:
		MUTEX_DEBUG(("Lock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
		break;
	default:
		MUTEX_DEBUG(("Lock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Async lock. If can not lock, return false Linux
 * @param  none
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::AsyncLock(
)
{
	int errorCode;

	if(m_created == false)
	{
		MUTEX_DEBUG(("AsyncLock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	errorCode = pthread_mutex_trylock(&m_mutex);

	switch(errorCode)
	{
	case 0:
		MUTEX_DEBUG(("AsyncLock success : m_mutex = %d\n", &m_mutex));
		return true;
		break;
	case EINVAL:
		MUTEX_DEBUG(("AsyncLock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
		break;
	case EBUSY :
	default:
		MUTEX_DEBUG(("AsyncLock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Timed lock. If can not lock, wait until times up for Linux
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::TimedLock(
	unsigned long time ///< [in] Timeout time[usec]
)
{
	unsigned long progress;
	int errorCode;
	struct timeval start;
	struct timeval end;

	if(m_created == false)
	{
		MUTEX_DEBUG(("TimedLock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	/* Get wait start time */
	gettimeofday(&start, NULL);

	while(1)
	{
		errorCode = pthread_mutex_trylock(&m_mutex);

		switch(errorCode)
		{
		case 0:
			MUTEX_DEBUG(("TimedLock success : m_mutex = %d\n", &m_mutex));
			return true;
			break;
		case EINVAL:
			MUTEX_DEBUG(("TimedLock failed : m_mutex = %d\n", &m_mutex));
			m_error = ERROR_NOMUTEX;
			return false;
			break;
		case EBUSY :
		default:
			break;
		}

		/* Get progress time */
		gettimeofday(&end, NULL);
		progress = (end.tv_sec - start.tv_sec) * 1000 + (end.tv_usec - start.tv_usec) / 1000;
		if(progress > time)
		{
			MUTEX_DEBUG(("TimedLock failed : m_mutex = %d\n", &m_mutex));
			m_error = ERROR_TIME_OUT;
			return false;
		}
	}

	return false;
}

/**
 * @brief  Unlock. If can not unlock, return false for Linux
 * @param  none
 * @return Unlock success or not
 * @retval true:Unlock success
 * @retval false:Unlock failed
 */
bool PafeeMutex::Unlock(
)
{
	int returnValue;

	if(m_created == false)
	{
		MUTEX_DEBUG(("Unlock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
	}

	returnValue = pthread_mutex_unlock(&m_mutex);

	switch(returnValue)
	{
	case 0:
		MUTEX_DEBUG(("Unlock success : m_mutex = %d\n", &m_mutex));
		return true;
		break;

	case EINVAL:
		MUTEX_DEBUG(("Unlock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_NOMUTEX;
		return false;
		break;

	case EPERM:
	default:
		MUTEX_DEBUG(("Unlock failed : m_mutex = %d\n", &m_mutex));
		m_error = ERROR_CANNOT_UNLOCK;
		return false;
		break;
	}
}

/**
 * @brief  Get error code. 
 * @param  none
 * @return Mutex error code
 * @retval MutexError
 */
PafeeMutex::MUTEX_ERROR PafeeMutex::GetLastError(
)
{
	return m_error;
}

#endif // PAFEE_OS_LINUX

#ifdef PAFEE_OS_ITRON
int MutexSyslog(const char *string, ...)
{
	va_list varList;

	va_start(varList, string);

	return syslog(LOG_NOTICE, string, *varList);
}

/**
 * @brief  Mutex class constructor for Itron
 * @param  none
 * @return none
 */
PafeeMutex::PafeeMutex(
)
{
	m_id = 0;
	m_error = ERROR_NONE;

	MUTEX_DEBUG(("Constractor\n"));
}

/**
 * @brief  Mutex class destructor
 * @param  none
 * @return none
 */
PafeeMutex::~PafeeMutex(
)
{
	Delete();

	MUTEX_DEBUG(("Destractor\n"));
}

/**
 * @brief  Attach static mutex id for Windows
 * @return Attach success or not
 * @retval true:create success
 * @retval false:create failed
 */
bool PafeeMutex::Attach(
	ID id ///< [in] Mutex ID
)
{
	if(m_id != 0)
	{
		m_error = ERROR_HAS_MUTEX;

		MUTEX_DEBUG(("Attach failed : m_id = %d\n", m_id));

		return false;
	}
	else
	{
		m_id = id;

		MUTEX_DEBUG(("Attach success : m_id = %d\n", m_id));

		return true;
	}
}

/**
 * @brief  Free static mutex id for Itron
 * @param  none
 * @return Delete success or not
 * @retval true:Delete success
 * @retval false:Delete failed
 */
bool PafeeMutex::Delete(
)
{
	m_id = 0;

	MUTEX_DEBUG(("Delete success : m_id = %d\n", m_id));

	return true;
}

/**
 * @brief  Lock. If can not lock, wait until get lock for Itron
 * @param  none
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::Lock(
)
{
	ER errorCode;

	if(m_id == 0)
	{
		m_error = ERROR_NOMUTEX;

		MUTEX_DEBUG(("Lock failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = wai_sem(m_id);

	switch(errorCode)
	{
	case E_OK:
		MUTEX_DEBUG(("Lock success : m_id = %d\n", m_id));
		return true;
		break;

	case E_TMOUT:
		MUTEX_DEBUG(("Lock failed : m_id = %d\n", m_id));
		m_error = ERROR_TIME_OUT;
		return false;
		break;

#if 0
	case E_NOEXS:
	case E_ID:
		MUTEX_DEBUG(("Lock failed : m_id = %d\n", m_id));
		m_error = ERROR_NOMUTEX;
		return false;
		break;
#endif

	case E_NOEXS:
	case E_ID:
	case E_PAR:
	case E_RLWAI:
	case E_DLT:
	default:
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Async lock. If can not lock, return false for Itron
 * @param  none
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::AsyncLock(
)
{
	ER errorCode;

	if(m_id == 0)
	{
		m_error = ERROR_NOMUTEX;

		MUTEX_DEBUG(("Lock failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = pol_sem(m_id);

	switch(errorCode)
	{
	case E_OK:
		MUTEX_DEBUG(("AsyncLock success : m_id = %d\n", m_id));
		return true;
		break;

#if 0
	case E_NOEXS:
	case E_ID:
		MUTEX_DEBUG(("AsyncLock failed : m_id = %d\n", m_id));
		m_error = ERROR_NOMUTEX;
		return false;
		break;
#endif

	case E_TMOUT:
#if 0
		MUTEX_DEBUG(("AsyncLock failed : m_id = %d\n", m_id));
		m_error = ERROR_TIME_OUT;
		return false;
		break;
#endif
	case E_NOEXS:
	case E_ID:
	case E_PAR:
	case E_RLWAI:
	case E_DLT:
	default:
		MUTEX_DEBUG(("AsyncLock failed : m_id = %d\n", m_id));
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Timed lock. If can not lock, wait until times up for Itron
 * @return Lock success or not
 * @retval true:Lock success
 * @retval false:Lock failed
 */
bool PafeeMutex::TimedLock(
	unsigned long time ///< [in] Timeout time
)
{
	ER errorCode;

	if(m_id == 0)
	{
		m_error = ERROR_NOMUTEX;

		MUTEX_DEBUG(("Lock failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = twai_sem(m_id, time);

	switch(errorCode)
	{
	case E_OK:
		MUTEX_DEBUG(("TimedLock success : m_id = %d\n", m_id));
		return true;
		break;

	case E_TMOUT:
		MUTEX_DEBUG(("TimedLock failed : m_id = %d\n", m_id));
		m_error = ERROR_TIME_OUT;
		return false;
		break;

#if 0
	case E_NOEXS:
	case E_ID:
		MUTEX_DEBUG(("TimedLock failed : m_id = %d\n", m_id));
		m_error = ERROR_NOMUTEX;
		return false;
		break;
#endif

	case E_NOEXS:
	case E_ID:
	case E_PAR:
	case E_RLWAI:
	case E_DLT:
	default:
		MUTEX_DEBUG(("TimedLock failed : m_id = %d\n", m_id));
		m_error = ERROR_CANNOT_LOCK;
		return false;
		break;
	}
}

/**
 * @brief  Unlock. If can not unlock, return false for Itron
 * @param  none
 * @return Unlock success or not
 * @retval true:Unlock success
 * @retval false:Unlock failed
 */
bool PafeeMutex::Unlock(
)
{
	ER errorCode;

	if(m_id == 0)
	{
		m_error = ERROR_NOMUTEX;

		MUTEX_DEBUG(("Lock failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = sig_sem(m_id);

	switch(errorCode)
	{
	case E_OK:
		MUTEX_DEBUG(("Unlock success : m_id = %d\n", m_id));
		return true;
		break;

#if 0
	case E_NOEXS:
	case E_ID:
		MUTEX_DEBUG(("Unlock failed : m_id = %d\n", m_id));
		m_error = ERROR_NOMUTEX;
		return false;
		break;
#endif

	case E_NOEXS:
	case E_ID:
	case E_QOVR:
	default:
		MUTEX_DEBUG(("Unlock failed : m_id = %d\n", m_id));
		m_error = ERROR_CANNOT_UNLOCK;
		return false;
		break;
	}
}

/**
 * @brief  Get error code. 
 * @param  none
 * @return Mutex error code
 * @retval MutexError
 */
PafeeMutex::MUTEX_ERROR PafeeMutex::GetLastError(
)
{
	return m_error;
}

#endif // PAFEE_OS_ITRON

#ifdef __cplusplus
}; // namespace Pafee
#endif
