//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	Mutex.h
//! 	 @author:	Technocrats
//! 	 @brief:	Mutex class header
//
//********************************************************************
//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND noneINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _PAFEE_MUTEX_H_INCLUDED_
#define _PAFEE_MUTEX_H_INCLUDED_

#ifdef PAFEE_OS_WINDOWS
#include <windows.h>
#endif

#ifdef PAFEE_OS_LINUX
#include <pthread.h>
#endif

#ifdef PAFEE_OS_ITRON
#include <kernel.h>
#endif

#ifdef PAFEE_DEBUG
#include <stdio.h>
#endif

#ifdef __cplusplus
namespace Pafee{
#endif

#if defined(PAFEE_OS_WINDOWS) || defined(PAFEE_OS_LINUX) || defined(PAFEE_OS_ITRON)

#ifdef PAFEE_DEBUG
#ifdef PAFEE_OS_ITRON
#define MUTEX_DEBUG(STRING) MutexSyslog STRING
#else
#define	MUTEX_DEBUG(STRING) printf STRING
#endif
#else
#define MUTEX_DEBUG(STRING)
#endif

#ifdef PAFEE_OS_ITRON
int MutexSyslog(const char *string, ...);
#endif
class PafeeMutex{
public:
	/**
	 * @brief  Mutex error emun
	 */
	typedef enum tagMUTEX_ERROR{
		ERROR_NONE, /**< Error none */
		ERROR_NOMUTEX, /**< handle, id is not exist */
		ERROR_HAS_MUTEX, /**< Allready exist handle */
		ERROR_CANNOT_CREATE, /**< Mutex can not create */
		ERROR_CANNOT_DELETE, /**< Mutex can not delete */
		ERROR_CANNOT_LOCK, /**< Mutex can not get */
		ERROR_TIME_OUT, /**< Mutex can not get for time out */
		ERROR_CANNOT_UNLOCK, /**< Error none */
		ERROR_OHTER, /**< Unknown error */
		ERROR_MAX /**< Error none */
	} MUTEX_ERROR;

private:
#ifdef PAFEE_OS_WINDOWS
	HANDLE m_handle; /**< Windows mutex handle */
#endif // PAFEE_OS_WINDOWS

#ifdef PAFEE_OS_LINUX
	pthread_mutex_t m_mutex; /**< Linux mutex handle */
	bool m_created; /**< Mutex object created flag */
#endif // PAFEE_OS_LINUX

#ifdef PAFEE_OS_ITRON
	ID m_id; /**< Itron mutex handle */
#endif // PAFEE_OS_ITRON

	MUTEX_ERROR m_error;

public:
	/**
	 * @brief  PafeeMutex class constructor
	 * @param  none
	 * @return none
	 */
	PafeeMutex();

	/**
	 * @brief  PafeeMutex class destructor
	 * @param  none
	 * @return none
	 */
	~PafeeMutex();

#if defined(PAFEE_OS_WINDOWS) || defined(PAFEE_OS_LINUX)
	/**
	 * @brief  Make new mutex
	 * @param  none
	 * @return Create success or not
	 * @retval true:create success
	 * @retval false:create failed
	 */
	bool Create();
#endif

#ifdef PAFEE_OS_ITRON
	/**
	 * @brief  Attach static mutex id
	 * @return Attach success or not
	 * @retval true:create success
	 * @retval false:create failed
	 */
	bool Attach(
		ID id ///< [in] Mutex ID
	);
#endif

	/**
	 * @brief  Delete mutex or Free static mutex id
	 * @param  none
	 * @return Delete success or not
	 * @retval true:Delete success
	 * @retval false:Delete failed
	 */
	bool Delete();

	/**
	 * @brief  Lock. If can not lock, wait until get lock
	 * @param  none
	 * @return Lock success or not
	 * @retval true:Lock success
	 * @retval false:Lock failed
	 */
	bool Lock();

	/**
	 * @brief  Async lock. If can not lock, return false
	 * @param  none
	 * @return Lock success or not
	 * @retval true:Lock success
	 * @retval false:Lock failed
	 */
	bool AsyncLock();

	/**
	 * @brief  Timed lock. If can not lock, wait until times up
	 * @return Lock success or not
	 * @retval true:Lock success
	 * @retval false:Lock failed
	 */
	bool TimedLock(
		unsigned long time ///< [in] timeout time
	);

	/**
	 * @brief  Unlock. If can not unlock, return false
	 * @param  none
	 * @return Unlock success or not
	 * @retval true:Unlock success
	 * @retval false:Unlock failed
	 */
	bool Unlock();

	/**
	 * @brief  Get m_error code. 
	 * @param  none
	 * @return Mutex error code
	 * @retval MutexError
	 */
	MUTEX_ERROR GetLastError();
}; // class Mutex
#endif // defined(PAFEE_OS_WINDOWS) || defined(PAFEE_OS_LINUX) || defined(PAFEE_OS_ITRON)

#ifdef __cplusplus
}; // namespace Pafee
#endif

#endif // _PAFEE_MUTEX_H_INCLUDED_
