//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	Signal.cpp
//! 	 @author:	Technocrats
//! 	 @brief:	Signal class source
//
//********************************************************************
//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND noneINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "PafeeSignal.h"
#include <limits.h>

#ifdef PAFEE_OS_LINUX
#include <sys/time.h>
#include <sys/errno.h>
#endif

namespace Pafee{

#ifdef PAFEE_OS_WINDOWS
/**
 * @brief  Signal class constructor for Windows
 * @param  none
 * @return none
 */
PafeeSignal::PafeeSignal()
{
	m_handle = INVALID_HANDLE_VALUE;
	m_error = ERROR_NONE;
}
/**
 * @brief  Signal class destructor for Windows
 * @param  none
 * @return none
 */
PafeeSignal::~PafeeSignal()
{
	Delete();
}

/**
 * @brief  Make new signal for Windows
 * @param  none
 * @return Create success or not
 * @retval true:create success
 * @retval false:create failed
 */
bool PafeeSignal::Create()
{
	if(m_handle != INVALID_HANDLE_VALUE)
	{
		m_error = ERROR_HAS_SIGNAL;
		return false;
	}

	m_handle = CreateEvent(
					NULL, // Default Seculity
					false, // Auto reset only
					false, // Initialize not signal
					NULL // No name event
					);

	if(m_handle == NULL)
	{
		m_error = ERROR_CANNOT_CREATE;
		return false;
	}
	else
	{
		return true;
	}
}

/**
 * @brief  Delete signal for Windows
 * @param  none
 * @return Delete success or not
 * @retval true:Delete success
 * @retval false:Delete failed
 */
bool PafeeSignal::Delete()
{
	int returnValue;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}
	else
	{
		returnValue = CloseHandle(m_handle);

		if(returnValue == 0)
		{
			m_error = ERROR_CANNOT_DELETE;
			return false;
		}
		else
		{
			m_handle = INVALID_HANDLE_VALUE;
			return true;
		}
	}
}

/**
 * @brief  Wait. If signal was not send, wait until get signal for Windows
 * @param  none
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::Wait()
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}

	errorCode = WaitForSingleObject(m_handle, INFINITE);
	switch(errorCode)
	{
	case WAIT_OBJECT_0:
	case WAIT_ABANDONED:
		return true;
		break;

	case WAIT_TIMEOUT:
		m_error = ERROR_TIME_OUT;
		return false;
		break;
	case WAIT_FAILED:
	default:
		m_error = ERROR_CANNOT_SIGNAL;
		return false;
		break;
	}
}

/**
 * @brief  Asyncronyze Wait. If signal was not send, return false for Windows
 * @param  none
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::AsyncWait()
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}

	errorCode = WaitForSingleObject(m_handle, 0);
	switch(errorCode)
	{
	case WAIT_OBJECT_0:
	case WAIT_ABANDONED:
		return true;
		break;

	case WAIT_TIMEOUT:
#if 0
		m_error = ERROR_TIME_OUT;
		return false;
		break;
#endif
	case WAIT_FAILED:
	default:
		m_error = ERROR_CANNOT_SIGNAL;
		return false;
		break;
	}
}

/**
 * @brief  Timed wait. If signal was not send, wait until times up for Windows
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::TimedWait(
	unsigned long timeout ///< [in] timeout time
)
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}
	else
	{
		errorCode = WaitForSingleObject(m_handle, timeout);
		switch(errorCode)
		{
		case WAIT_OBJECT_0:
		case WAIT_ABANDONED:
			return true;
			break;

		case WAIT_TIMEOUT:
			m_error = ERROR_TIME_OUT;
			return false;
			break;
		case WAIT_FAILED:
		default:
			m_error = ERROR_CANNOT_SIGNAL;
			return false;
			break;
		}
	}
}

/**
 * @brief  Send signal for Windows
 * @param  none
 * @return none
 */
bool PafeeSignal::Send()
{
	unsigned long errorCode;

	if(m_handle == INVALID_HANDLE_VALUE)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}
	else
	{
		errorCode = SetEvent(m_handle);
		if(errorCode == 0)
		{
			m_error = ERROR_CANNOT_SET_EVENT;
			return false;
		}
#if 0
		else
		{
			errorCode = ResetEvent(m_handle);
			if(errorCode == 0)
			{
				m_error = ERROR_CANNOT_RESET_EVENT;
				return false;
			}
			else
			{
				return true;
			}
		}
#else
		else
		{
			return true;
		}
#endif
	}
}

/**
 * @brief  Get error code for Windows
 * @param  none
 * @return Signal error code
 * @retval SignalError
 */
PafeeSignal::SIGNAL_ERROR PafeeSignal::GetLastError()
{
	return m_error;
}

#endif // PAFEE_OS_WINDOWS

#ifdef PAFEE_OS_LINUX
/**
 * @brief  Signal class constructor for Linux
 * @param  none
 * @return none
 */
PafeeSignal::PafeeSignal()
{
	m_created = false; 
	m_signal = false;
	m_error = ERROR_NONE;
}

/**
 * @brief  Signal class destructor for Linux
 * @param  none
 * @return none
 */
PafeeSignal::~PafeeSignal()
{
	Delete();
}

/**
 * @brief  Make new signal for Linux
 * @param  none
 * @return Create success or not
 * @retval true:create success
 * @retval false:create failed
 */
bool PafeeSignal::Create()
{
	if(m_created == true)
	{
		m_error = ERROR_HAS_SIGNAL;
		return false;
	}

	pthread_mutex_init(&m_mutex, NULL); // default : fast
	pthread_cond_init(&m_condition, NULL); // default condition
	m_signal = false;

	m_created = true;
	return true;
}

/**
 * @brief  Delete signal for Linux
 * @param  none
 * @return Delete success or not
 * @retval true:Delete success
 * @retval false:Delete failed
 */
bool PafeeSignal::Delete()
{
	int returnValue;

	if(m_created == false)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}

	returnValue = pthread_cond_destroy(&m_condition);

	switch(returnValue)
	{
	case 0:
		m_created = false;
		m_signal = false;
		returnValue = pthread_mutex_destroy(&m_mutex);
		return true;
		break;
	case EBUSY:
	default:
		m_error = ERROR_CANNOT_DELETE;
		return false;
		break;
	}
}

/**
 * @brief  Wait. If signal was not send, wait until get signal for Linux
 * @param  none
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::Wait()
{
	if(m_created == false)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}
	pthread_mutex_lock(&m_mutex);
	if(m_signal == true)
	{
	}
	else
	{
		pthread_cond_wait(&m_condition, &m_mutex);
	}
	m_signal = false;
	pthread_mutex_unlock(&m_mutex);

	return true;
}

/**
 * @brief  Asyncronyze Wait. If signal was not send, return false for Linux
 * @param  none
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::AsyncWait()
{
	struct timespec timeup;
	int errorCode;

	if(m_created == false)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}

	timeup.tv_sec = time(NULL);
	timeup.tv_nsec = 0;
	pthread_mutex_lock(&m_mutex);
	if(m_signal == true)
	{
		errorCode = 0;
	}
	else
	{
		errorCode = pthread_cond_timedwait(&m_condition, &m_mutex, &timeup);
	}
	m_signal = false;
	pthread_mutex_unlock(&m_mutex);

	switch(errorCode)
	{
	case 0:
		return true;
		break;

	case EINTR:
		return true;
		break;

	case ETIMEDOUT:
	default:
		m_error = ERROR_CANNOT_SIGNAL;
		return false;
		break;
	}
}

/**
 * @brief  Timed wait. If signal was not send, wait until times up for Linux
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::TimedWait(
	unsigned long timeout ///< [in] timeout time[usec]
)
{
	struct timespec timeup;
	struct timeval now;
	unsigned long errorCode;

	if(m_created == false)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}

	timeup.tv_sec = time(NULL) + timeout / 1000000;
	timeup.tv_nsec = timeout % 1000000 * 1000;

	pthread_mutex_lock(&m_mutex);
	if(m_signal == true)
	{
		errorCode = 0;
	}
	else
	{
		errorCode = pthread_cond_timedwait(&m_condition, &m_mutex, &timeup);
	}
	m_signal = false;
	pthread_mutex_unlock(&m_mutex);

	switch(errorCode)
	{
	case 0:
		return true;
		break;

	case ETIMEDOUT:
		m_error = ERROR_TIME_OUT;
		return false;
		break;

	case EINTR:
		return true;
		break;

	default:
		m_error = ERROR_CANNOT_SIGNAL;
		return false;
		break;
	}
}

/**
 * @brief  Send signal for Linux
 * @param  none
 * @return none
 */
bool PafeeSignal::Send()
{
	unsigned long errorCode;

	if(m_created == false)
	{
		m_error = ERROR_NOSIGNAL;
		return false;
	}

	pthread_mutex_lock(&m_mutex);
	pthread_cond_signal(&m_condition);
	m_signal = true;
	pthread_mutex_unlock(&m_mutex);

	return true;
}

/**
 * @brief  Get error code for Linux
 * @param  none
 * @return Signal error code
 * @retval SignalError
 */
PafeeSignal::SIGNAL_ERROR PafeeSignal::GetLastError()
{
	return m_error;
}
#endif // PAFEE_OS_LINUX

#ifdef PAFEE_OS_ITRON

int SignalSyslog(const char *string, ...)
{
	va_list varList;

	va_start(varList, string);

	return syslog(LOG_NOTICE, string, *varList);
}

#define SIGNAL_FLAG_SET 0x01
#define SIGNAL_FLAG_CLEAR 0x00


/**
 * @brief  Signal class constructor for Itron
 * @param  none
 * @return none
 */
PafeeSignal::PafeeSignal()
{
	m_id = 0;
	m_error = ERROR_NONE;
	SIGNAL_DEBUG(("Constructor"));
}

/**
 * @brief  Signal class destructor for Itron
 * @param  none
 * @return none
 */
PafeeSignal::~PafeeSignal()
{
	Delete();
	SIGNAL_DEBUG(("Destructor"));
}

/**
 * @brief  Attach static signal id
 * @return Attach success or not
 * @retval true:create success
 * @retval false:create failed
 */
bool PafeeSignal::Attach(
	ID id ///< [in] Signal ID
)
{
	if(m_id != 0)
	{
		m_error = ERROR_HAS_SIGNAL;

		SIGNAL_DEBUG(("Attach failed m_id = %d\n", m_id));

		return false;
	}

	m_id = id;

	SIGNAL_DEBUG(("Attach success m_id = %d\n", m_id));

	return true;
}

/**
 * @brief  Delete signal for Itron
 * @param  none
 * @return Delete success or not
 * @retval true:Delete success
 * @retval false:Delete failed
 */
bool PafeeSignal::Delete()
{
	if(m_id == 0)
	{
		m_error = ERROR_NOSIGNAL;

		SIGNAL_DEBUG(("Delete failed m_id = %d\n", m_id));

		return false;
	}

#if 0
	returnValue = del_flg(m_id);

	switch(returnValue)
	{
	case E_OK:
		return true;
		break;
	case E_ID:
	case E_NOEXS:
	default:
		m_error = ERROR_CANNOT_DELETE;
		return false;
		break;
	}
#endif
	SIGNAL_DEBUG(("Delete success m_id = %d\n", m_id));

	m_id = 0;

	return true;
}

/**
 * @brief  Wait. If signal was not send, wait until get signal for Itron
 * @param  none
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::Wait()
{
	ER errorCode;
	FLGPTN flagPattern;

	if(m_id == 0)
	{
		m_error = ERROR_NOSIGNAL;

		SIGNAL_DEBUG(("Wait failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = wai_flg(
		m_id, // id
		SIGNAL_FLAG_SET,  // wait flag pattern
		TWF_ANDW, // finish wait, clear flag
		&flagPattern // finish flag pattern
	);

	switch(errorCode)
	{
	case E_OK:
		SIGNAL_DEBUG(("Wait success m_id = %d\n", m_id));
		return true;
		break;

	case E_TMOUT:
		m_error = ERROR_TIME_OUT;

		SIGNAL_DEBUG(("Wait failed m_id = %d\n", m_id));

		return false;
		break;

	case E_ID:
	case E_NOEXS:
	case E_PAR:
	case E_ILUSE:
	case E_RLWAI:
	case E_DLT:
	default:
		m_error = ERROR_CANNOT_SIGNAL;

		SIGNAL_DEBUG(("Wait failed m_id = %d\n", m_id));

		return false;
		break;
	}

	return true;
}

/**
 * @brief  Asyncronyze Wait. If signal was not send, return false for Itron
 * @param  none
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::AsyncWait()
{
	ER errorCode;
	FLGPTN flagPattern;

	if(m_id == 0)
	{
		m_error = ERROR_NOSIGNAL;

		SIGNAL_DEBUG(("Asyncwait failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = pol_flg(
		m_id, // id
		SIGNAL_FLAG_SET,  // wait flag pattern
		TWF_ANDW, // finish wait, clear flag
		&flagPattern // finish flag pattern
	);

	switch(errorCode)
	{
	case E_OK:
		SIGNAL_DEBUG(("Asyncwait success m_id = %d\n", m_id));
		return true;
		break;

	case E_TMOUT:
		m_error = ERROR_TIME_OUT;

		SIGNAL_DEBUG(("Asyncwait failed m_id = %d\n", m_id));

		return false;
		break;

	case E_ID:
	case E_NOEXS:
	case E_PAR:
	case E_ILUSE:
	case E_RLWAI:
	case E_DLT:
	default:
		m_error = ERROR_CANNOT_SIGNAL;

		SIGNAL_DEBUG(("ER = %d\n", errorCode));
		SIGNAL_DEBUG(("Asyncwait failed m_id = %d\n", m_id));

		return false;
		break;
	}

	return true;
}

/**
 * @brief  Timed wait. If signal was not send, wait until times up for Itron
 * @return Get signal or not
 * @retval true:Get signal
 * @retval false:Not get signal
 */
bool PafeeSignal::TimedWait(
	unsigned long timeout ///< [in] timeout time[usec]
)
{
	ER errorCode;
	FLGPTN flagPattern;

	if(m_id == 0)
	{
		m_error = ERROR_NOSIGNAL;

		SIGNAL_DEBUG(("Timedwait failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = twai_flg(
		m_id, // id
		SIGNAL_FLAG_SET,  // wait flag pattern
		TWF_ANDW, // finish wait, clear flag
		&flagPattern, // finish flag pattern
		timeout // wait time
	);

	switch(errorCode)
	{
	case E_OK:
		SIGNAL_DEBUG(("Timedwait success m_id = %d\n", m_id));
		return true;
		break;

	case E_TMOUT:
		m_error = ERROR_TIME_OUT;

		SIGNAL_DEBUG(("Timedwait failed m_id = %d\n", m_id));

		return false;
		break;

	case E_ID:
	case E_NOEXS:
	case E_PAR:
	case E_ILUSE:
	case E_RLWAI:
	case E_DLT:
	default:
		m_error = ERROR_CANNOT_SIGNAL;

		SIGNAL_DEBUG(("ER = %d\n", errorCode));
		SIGNAL_DEBUG(("Timedwait failed m_id = %d\n", m_id));

		return false;
		break;
	}

	return true;
}

/**
 * @brief  Send signal for Itron
 * @param  none
 * @return none
 */
bool PafeeSignal::Send()
{
	ER errorCode;

	if(m_id == 0)
	{
		m_error = ERROR_NOSIGNAL;

		SIGNAL_DEBUG(("Send failed m_id = %d\n", m_id));

		return false;
	}

	errorCode = set_flg(m_id, SIGNAL_FLAG_SET);

	switch(errorCode)
	{
	case E_OK:
		SIGNAL_DEBUG(("Send success m_id = %d\n", m_id));
		return true;
		break;

	case E_ID:
	case E_NOEXS:
	case E_PAR:
	default:
		m_error = ERROR_CANNOT_SIGNAL;

		SIGNAL_DEBUG(("Send failed m_id = %d\n", m_id));

		return false;
		break;
	}

	return true;
}

/**
 * @brief  Get error code for Itron
 * @param  none
 * @return Signal error code
 * @retval SignalError
 */
PafeeSignal::SIGNAL_ERROR PafeeSignal::GetLastError()
{
	return m_error;
}
#endif // PAFEE_OS_ITRON

}; // namespace Pafee
