//****************************************************
//
//  project     PAFEE
//! @file:      PafeeFile.h
//! @author:    Xie Jian
//! @brief:     Supply file access interface
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//


#ifdef ENV_WIN32
#pragma warning(disable:4996)
#endif

#include <stdio.h>
#include "PafeeCommon.h"
#include "PafeeObject.h"

#ifndef _FILE_H_INCLUDED_
#define _FILE_H_INCLUDED_

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{

static const char MODENAME[][4] =
{
	"r",
	"w",
	"a",
	"r+",
	"w+",
	"a+",
	"rb",
	"wb",
	"ab",
	"r+b",
	"w+b",
	"a+b"
};

//------------------------------------------------------------------------------
//!    - @brief    Supply file access interface; Expand from C standard function
//!        - file open
//!        - file close
//!        - file input
//!        - file output
//!        - file flash
//!        - file seek
//!    .
//------------------------------------------------------------------------------
//! File
class File : public Object
{

public:
	enum
	{
		TOP, 
		CURRENT,
		END
	} ORIGIN;

	enum OPENMODE
	{
		MODE_READ=0,
		MODE_WRITE=1,
		MODE_APPEND=2,
		MODE_READPLUS=3,
		MODE_WRITEPLUS=4,
		MODE_APPENDPLUS=5,
		//binary
		MODE_READ_BINARY=6,
		MODE_WRITE_BINARY=7,
		MODE_APPEND_BINARY=8,
		MODE_READPLUS_BINARY=9,
		MODE_WRITEPLUS_BINARY=10,
		MODE_APPENDPLUS_BINARY=11
	};

public:
	File();
	~File();
	//
	bool Open(const char* filePath, const OPENMODE mode);//File::OPENMODE
	void Close(void);
	long GetLength(void);
	//-------------------------------------------------------------------------
	//!	Get the length of data that really read out. 
	//!	@return Result(the real length of the data that to be read out from the file)
	//-------------------------------------------------------------------------
	inline int Read(
		void* pBuffer, /// < [out] The data which were read
		int length /// < [in] Length to read
	)
	{
		return (int)fread(pBuffer, 1, length, m_pFile);
	}
	//-------------------------------------------------------------------------
	//!	Get the length of data that really writed in. 
	//!	@return Result(the real length of the data that to be write into the file)
	//-------------------------------------------------------------------------
	inline int Write(
		const void* pData, /// < [in] Data to write
		int length /// < [in] Length to write
	)
	{
		return (int)fwrite(pData, 1,length, m_pFile);
	}
	//-------------------------------------------------------------------------
	//!	push the data in buffer into the file(write)/(read)
	//-------------------------------------------------------------------------
	inline void Flush(void)
	{
		fflush(m_pFile);
	}
	bool Seek(int offset, int origin);//File::ORIGIN 

private:
	FILE *m_pFile;
	OPENMODE m_localOpenMode;

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest(void);
#endif // PAFEE_UNITTEST
};

}
#endif //_FILE_H_INCLUDED_
